# routines to perform unfolding of the Brillouin zone

import numpy as np
from scipy.linalg import eigh

def unfolded_bands(hfol,hprim,kpath,inds_super=[]):
  """ Save in file unfolded band structure"""
  hkfol = hfol.get_hk_gen()  # generator
  hkprim = hprim.get_hk_gen() # generator
  nump = hprim.intra.shape[0] # number orbitals of primitive
  numf = hfol.intra.shape[0] # number orbitals of folded
  nn = numf/numfp # number times the hamiltonian is bigger
  for k in kpath:
    kfol = k # kpoint
    kprim = k/3.
    hf = hkfol(k) # get matrix
    hp = hkprim(k) # get matrix
    (ep,wfp) = eigh(hp) # eigenvalues and eigenvectors 
    (ef,wff) = eigh(hf) # eigenvalues and eigenvectors 
    dp = [(w.dot(np.conjugate(w))).real for w in wfp.transpose()] # density prim
    df = [(w.dot(np.conjugate(w))).real for w in wff.transpose()] # density fol
    df = [sum(w.split(nn)) for w in df] # transform into smaller basis
    # now it is time to compare weights of each eigenvalue
    raise





def perturb_bands(hprim,hper,kpath,inds_super=[]):
  """ Save in file perturbed band structure"""
  hkprim = hprim.get_hk_gen() # generator
  for k in kpath: # loop over kpoints
    kprim = k/3.
    hp = hkprim(k) # get matrix
    (ep,wfp) = eigh(hp) # eigenvalues and eigenvectors 
    wfp = wfp.transpose() # transpose eigenvectors
    # now calculate eigenfunctions in a supercell
    wfs = [] # wavefunction of the supercell
    for w in wfp: # loop over functions
      wlist = [] # create empty list
      for ix in range(inds_super[0]): # loop over ix
        for iy in range(inds_super[1]):  # loop over iy
          for iz in range(inds_super[2]):  # loop over iz
            phi = k.dot(np.array([ix,iy,iz])) # phase
            wj = w*np.exp(1j*np.pi*phi) # wavefunction
            wlist.append(wj) # append to the list
      wfs.append(np.concatenate(wlist)) # add this wave to the list
   # now apply perturbation theory
    vm = np.zeros((len(wfs),len(wfs)),dtype=np.complex) # perturbation
    for iw in wfs: 
      for jw in wfs: 
        iw = np.matrix(iw)
        jw = np.matrix(jw).H
        vaw = (iw*hper*jw)[0,0] # matrix element
  
    raise



def bloch_projector(h,g0=None):
    """Given a certain Hamiltonian and minimal geometry, return
    a projector to the minimal BZ"""
    if g0 is None:
        if h.geometry.primal_geometry is None: 
            print("No primal geometry")
            raise 
        else: g0 = h.geometry.primal_geometry # get the primal geometry
    h0 = g0.get_hamiltonian(has_spin=False)
    if h.has_spin: h0.turn_spinful() # add spin
    if h.has_eh: h0.add_swave(0.0) # add electron hole
    if h.dimensionality<3: 
        from .supercell import infer_supercell
        nsuper = infer_supercell(h.geometry,g0) # get the supercell
    else: raise # not implemented
    fs = bloch_phase_matrix(h0,nsuper=nsuper)
    def fun(v,k=0):
        vo = np.conjugate(fs(k))@v # return vector
        out = np.abs(vo.dot(np.conjugate(vo))) # overlap
        return v*out
    from .operators import Operator
    return Operator(fun) # return operator



def bloch_phase_matrix_simple(self,nsuper=[1,1,1]):
    """Given a Hamiltonian, return the matrix with Bloch phases
    for a supercell. This is a simple non-optimal version
    of this function"""
    from scipy.sparse import bmat,csc_matrix
    if self.dimensionality>2: raise
    n = self.intra.shape[0] # dimensionality
    iden = csc_matrix(np.identity(n,dtype=np.complex)) # identity
    ns = nsuper[0]*nsuper[1] # number of supercells
    nx = nsuper[0] # supercells in x
    ny = nsuper[1] # supercells in y
    def fun(k): # generator
        out = [[None for i in range(ns)] ] 
#        for ii in range(ns): out[ii][ii] = 0*iden # initialize
        ii = 0 # counter
        for i in range(nx): # loop over x 
          for j in range(ny): # loop over y
            ki = np.array([k[0]/nx,k[1]/ny,0.]) # scale kvector
#            frac_r = self.geometry.frac_r # fractional coordinates
#            U = np.diag([self.geometry.bloch_phase(ki,r) for r in frac_r])
#            U = np.array(U) # this is without .H
#            U = np.conjugate(U).T
#            U = self.spinless2full(U) # increase the space if necessary
            phi = np.exp(1j*np.pi*2.*np.array([i,j,0]).dot(ki)) # phase
            out[0][ii] = iden*phi # multiply by phase
            ii += 1 # increase counter
        out = bmat(out) # return matrix
        return out
    return fun






def bloch_phase_matrix(self,nsuper=[1,1,1]):
    """Given a Hamiltonian, return the matrix with Bloch phases
    for a supercell"""
    from scipy.sparse import bmat,csc_matrix
    if self.dimensionality>2: raise
    n = self.intra.shape[0] # dimensionality
    iden = csc_matrix(np.identity(n,dtype=np.complex)) # identity
    ns = nsuper[0]*nsuper[1] # number of supercells
    nx = nsuper[0] # supercells in x
    ny = nsuper[1] # supercells in y
    outlist = [None for i in range(ns)] # list with matrices
    dlist = [None for i in range(ns)] # list with vectors
    ii = 0 # counter
    # generate all the matrices
    for i in range(nx): # loop over x 
      for j in range(ny): # loop over y
        out = [[None for ij in range(ns)]]
        for ij in range(ns): out[0][ij] = 0*iden # initialize
        dlist[ii] = np.array([i,j,0]) # list with dvectors
        out[0][ii] = iden*(1.+0j) # identity
        outlist[ii] = bmat(out).todense() # store dense matrix
        ii += 1 # increase counter
    dlist = np.array(dlist) # to array
    outlist = np.array(outlist) # to array
    def fun(k): # function generating the matrix
        ki = np.array([k[0]/nx,k[1]/ny,0.]) # scale kvector
        mout = outlist[0]*0j # initialize
        m = bloch_phase_matrix_jit(outlist,dlist,ki,mout)
        return m
    return fun

from numba import jit

@jit(nopython=True)
def bloch_phase_matrix_jit(ms,ds,k,out):
    """Numba function to generate the matrix"""
    n = len(ds) # loop over replicas
    out = out*0. # initialize
    for ii in range(n): # loop
        d = ds[ii]
        phi0 = d[0]*k[0] + d[1]*k[1] + d[2]*k[2]
        phi = np.exp(1j*np.pi*2.*phi0) # phase
        out = out + ms[ii]*phi # add contribution
    return out # return matrix



import numpy as np
from . import algebra

# compute RPA response
def chi_AB_RPA(h,V,**kwargs):
    """Compute the RPA chi for an SCF object"""
#    raise # not finished
    from .chi import chiAB # get response function
    chis = chiAB(h,**kwargs) # get the non-interacting response functions
    iden = np.identity(chis[0].shape[0],dtype=np.complex) # identity
    chis_rpa = [chi@algebra.inv(iden - V@chi) for chi in chis]
    return np.array(chis_rpa)


def chi_AB_RPA_scf(scf):
    """Return the RPA response function"""
    if len(scf.v)==1: # just the onsite term
        return chi_AB_RPA(scf.hamiltonian,scf.v[(0,0,0)])
    else: raise # not implemented




# library to perform massive green function calculations
import os
import numpy as np

from . import filesystem as fs

prec = 4



def round_folder(input_folder,output_folder,prec=prec):
  """Round the names of files in a certain folder"""
  names = os.listdir(os.getcwd()+"/"+input_folder)
  fs.rmdir(output_folder)
  fs.mkdir(output_folder)
  for ni in names:
    n = ni.split("_")
    pre = n[0]
    value = n[1].split(".dat")[0] # value
    value = round(float(value),prec) # value with new precision
    name = pre + "_"+str(value)+".dat"
    os.system("cp "+input_folder+"/"+ni+"  "+output_folder+"/"+name) # copy to the new




def clean():
  """ Clena all the directories"""
  os.system("rm -rf green_storage*")
  os.system("rm -rf pdos_storage*")


def get_green(fun_gf,name="",energy=0.0,prec=prec):
  """Looks for the green function at that energy, if it hasn't been calculated
  calculates it"""
  foldername = "green_storage_"+name # name of the foler
  dirname = os.getcwd()+"/"+foldername # name of the foler
  if not foldername in os.listdir(os.getcwd()):
    fs.mkdir(foldername) # create the folder if nonexistent
  er = round(energy,prec) # round the energy value
  namefile = "green_"+str(er)+".npy" # name of the file
  files = os.listdir(dirname) # files in the directory
  if namefile in files: # check if it has been calculated
    m = np.matrix(np.load(dirname+"/"+namefile)) # get the matrix
    return m # return the matrix
  else: # if it hasn't been calculated
    m = fun_gf(er) # calcualte green function
    np.save(dirname+"/"+namefile,m) # save the matrix
    return m # return the matrix



def get_pdos(fun_pd,name="",energy=0.0,prec=6):
  """Looks for the DOS at that energy, if it hasn't been calculated
  calculates it"""
  foldername = "pdos_storage_"+name # name of the foler
  dirname = os.getcwd()+"/"+foldername # name of the foler
  if not foldername in os.listdir(os.getcwd()):
    fs.mkdir(foldername) # create the folder if nonexistent
  er = round(energy,prec) # round the energy value
  namefile = "pdos_"+str(er)+".dat" # name of the file
  files = os.listdir(dirname) # files in the directory
  if namefile in files: # check if it has been calculated
    m = np.loadtxt(dirname+"/"+namefile) # get the matrix
    return m # return the matrix
  else: # if it hasn't been calculated
    m = fun_pd(er) # calcualte green function
    np.savetxt(dirname+"/"+namefile,m) # save the matrix
    return m # return the matrix





import numpy as np
from numba import jit

def selfconvolve(ds):
    """Do a selfconvolution"""
    if len(ds.shape)!=2: raise
    return selfconvolve_jit(ds,ds*0.0)


@jit(nopython=True)
def selfconvolve_jit(ds,out):
    """Convolve a 2D array with itself using periodic boundary conditions"""
    nx = ds.shape[0]
    ny = ds.shape[1]
    for i in range(nx):
      for j in range(ny):
        for ii in range(nx):
          for jj in range(ny):
              out[ii,jj] = ds[(i+ii)%nx,(j+jj)%nx]*ds[i,j] + out[ii,jj]
    return out

from .temperaturetk.convolution import temperature_convolution








from numba import jit
import numpy as np 



@jit(nopython=True)
def numba_number_first_neighbor(r1,r2):
   """Number of first neighbors"""
   n = 0
   for i in range(len(r1)):
     for j in range(len(r2)):
       ri = r1[i]
       rj = r2[j]
       dr = ri-rj
       dr2 = dr[0]*dr[0] + dr[1]*dr[1] + dr[2]*dr[2]
       if 0.8<dr2<1.2: n +=1
   return n # return neighbors
@jit(nopython=True)
def numba_pairs_first_neighbor(r1,r2,pairs):
   """Return the pairs"""
   n = 0
   for i in range(len(r1)):
     for j in range(len(r2)):
       ri = r1[i]
       rj = r2[j]
       dr = ri-rj
       dr2 = dr[0]*dr[0] + dr[1]*dr[1] + dr[2]*dr[2]
       if 0.8<dr2<1.2:
           pairs[n][0] = i
           pairs[n][1] = j
           n += 1 # increase counter
   return pairs
def find_first_neighbor(r1,r2):
    """Get first neighbors"""
    print("Using numba in find_first_neighbor")
    n = numba_number_first_neighbor(r1,r2) # number of first neighbors
    pairs = np.zeros((n,2),dtype=int) # create the array
    pairs = numba_pairs_first_neighbor(r1,r2,pairs) # get the pairs
    return pairs




from __future__ import print_function
from copy import deepcopy
from scipy.sparse import bmat
from scipy.sparse import csc_matrix as csc
import numpy as np
from . import sculpt

def honeycomb2square(h):
  """Transforms a honeycomb lattice into a square lattice"""
  ho = deepcopy(h) # output geometry
  g = h.geometry # geometry
  go = deepcopy(g) # output geometry
  go.a1 = - g.a1 - g.a2 
  go.a2 = g.a1 - g.a2
  # perform a check to see if the supercell is well built
  if np.abs(go.a1)[1]>0.001 or np.abs(go.a2[0])>0.001: raise
  go.x = np.concatenate([g.x,g.x-g.a1[0]])
  go.y = np.concatenate([g.y,g.y-g.a1[1]])
  go.z = np.concatenate([g.z,g.z])
  go.xyz2r() # update r atribbute
  zero = csc(h.tx*0.)
  # define sparse
  intra = csc(h.intra)
  tx = csc(h.tx)
  ty = csc(h.ty)
  txy = csc(h.txy)
  txmy = csc(h.txmy)
  # define new hoppings
  ho.intra = bmat([[intra,tx.H],[tx,intra]]).todense()
  ho.tx = bmat([[txy.H,zero],[ty.H,txy.H]]).todense()
  ho.ty = bmat([[txmy,ty.H],[txmy,zero]]).todense()
  ho.txy = bmat([[zero,None],[None,zero]]).todense()
  ho.txmy = bmat([[zero,zero],[tx.H,zero]]).todense()
  ho.geometry = go
  return ho



def invert_axis2(h):
  """Changes one axis in the hamiltonian"""
  ho = h.copy()
  ho.geometry.a2 = -h.geometry.a2
  ho.ty = h.ty.H
  ho.txy = h.txmy
  ho.txmy = h.txy
  return ho


def turn_square(h):
  """Turn the hamiltonian square no matter what!!!"""
  try: 
    return honeycomb2squareMoS2(h)
  except:
    ho = invert_axis2(h)
    return honeycomb2squareMoS2(ho)



def honeycomb2squareMoS2(h,check=True):
  """Transforms a honeycomb lattice into a square lattice"""
  ho = deepcopy(h) # output geometry
  g = h.geometry # geometry
  go = deepcopy(g) # output geometry
  go.a1 = g.a1 + g.a2 
  go.a2 = g.a1 - g.a2
  # now put the first vector along the x axis
  go.r = np.concatenate([g.r,g.r + g.a1])
  go.r2xyz()
  go = sculpt.rotate_a2b(go,go.a1,np.array([1.,0.,0.]))
  # perform a check to see if the supercell is well built  
  a1a2 = go.a1.dot(go.a2)
#  if a1a2>0.001: 
  if np.abs(go.a1)[1]>0.001 or np.abs(go.a2[0])>0.001: 
    ang = go.a1.dot(go.a2)
    print("The projection of lattice vectors is",ang)
    if check: raise
  go.r2xyz() # update r atribbute
  zero = csc(h.tx*0.)
  # define sparse
  intra = csc(h.intra)
  tx = csc(h.tx)
  ty = csc(h.ty)
  txy = csc(h.txy)
  txmy = csc(h.txmy)
  # define new hoppings
  ho.intra = bmat([[intra,tx],[tx.H,intra]]).todense()
  ho.tx = bmat([[txy,zero],[ty,txy]]).todense()
  ho.ty = bmat([[txmy,zero],[ty.H,txmy]]).todense()
  ho.txy = bmat([[zero,zero],[tx,zero]]).todense()
  ho.txmy = bmat([[zero,zero],[zero,zero]]).todense()
  ho.geometry = go
  return ho


















def bulk2ribbon(h,n=10):
  """Converts a hexagonal bulk hamiltonian into a ribbon hamiltonian"""
  from ribbonizate import hamiltonian_bulk2ribbon as b2r
  return b2r(h,n=n)  # workaround so that old script work














def bulk2ribbon_zz(h,n=10):
  """Converts a hexagonal bulk hamiltonian into a ribbon hamiltonian"""
  h = honeycomb2square(h) # create a square 2d geometry
  go = h.geometry.copy() # copy geometry
  ho = h.copy() # copy hamiltonian
  ho.dimensionality = 1 # reduce dimensionality
  go.dimensionality = 1 # reduce dimensionality
  intra = [[None for i in range(n)] for j in range(n)]
  inter = [[None for i in range(n)] for j in range(n)]
  for i in range(n): # to the the sam index
    intra[i][i] = csc(h.intra) 
    inter[i][i] = csc(h.tx) 
  for i in range(n-1): # one more or less
    intra[i][i+1] = csc(h.ty)  
    intra[i+1][i] = csc(h.ty.H)  
    inter[i+1][i] = csc(h.txmy) 
    inter[i][i+1] = csc(h.txy) 
  ho.intra = bmat(intra).todense()
  ho.inter = bmat(inter).todense()
  return ho




def orthogonalize_geometry(g):
  go = deepcopy(g) # output geometry
  go.a1 = g.a1 + g.a2
  go.a2 = g.a1 - g.a2
  # now put the first vector along the x axis
  go.r = np.concatenate([g.r,g.r + g.a1])
  go.r2xyz()
  go = sculpt.rotate_a2b(go,go.a1,np.array([1.,0.,0.]))
  # perform a check to see if the supercell is well built  
  a1a2 = go.a1.dot(go.a2)
#  if a1a2>0.001: 
  if np.abs(go.a1)[1]>0.001 or np.abs(go.a2[0])>0.001:
    ang = go.a1.dot(go.a2)
    print("The projection of lattice vectors is",ang)
    if check: raise
  go.has_sublattice = False
  go.r2xyz() # update r atribbute
  return go




from __future__ import print_function
import numpy as np

from . import extract

def swave(h,name="SWAVE.OUT",nrep=3):
  """Write the swave pairing of a Hamiltonian"""
  if not h.has_eh: raise
  d = h.extract("swave") # get the pairing
  g = h.geometry # get the geometry
  g.write_profile(np.abs(d),name="AMPLITUDE_"+name,
          normal_order=True,nrep=nrep)
  g.write_profile(np.angle(d)/(2*np.pi),name="PHASE_"+name,
          normal_order=True,nrep=nrep)
  return



  
  
def anomalous_hopping(h,name="ANOMALOUS_HOPPING.OUT",nrep=3,
        cutoff=1e-6):
    """Write in a file the s-wave hoppings"""
    from. import superconductivity
    if not h.has_eh: raise
    h = h.supercell(nrep)
    m = superconductivity.get_eh_sector_odd_even(h.intra,i=0,j=1)
    (ii,jj,ts) = extract.hopping_spinful(m)
    f = open(name,"w") # write file
    for (i,j,t) in zip(ii,jj,np.abs(ts)):
    #  if skip(h.geometry.r[i],h.geometry.r[j]): continue
      if t<cutoff: continue
      if i==j: continue
      f.write(str(h.geometry.r[i][0])+"  ")
      f.write(str(h.geometry.r[i][1])+"  ")
      f.write(str(h.geometry.r[j][0])+"  ")
      f.write(str(h.geometry.r[j][1])+"  ")
      f.write(str(np.round(t,4))+"\n")
    f.close()



def mz(h,name="MZ.OUT"):
  if h.has_eh: raise








def hopping(h,name="HOPPING.OUT",nrep=3,skip = lambda r1,r2: False,
        spin_imbalance=False,cutoff=1e-2,mode="abs"):
  """Write the magnitude of the hopping in a file"""
  if h.has_eh: 
      h = h.copy()
      h.remove_nambu()
  h = h.supercell(nrep)
  if h.has_spin: 
      if spin_imbalance:
          (ii,jj,ts) = extract.hopping_spinful_difference(h.intra,
                  skip_same_site=True)
      else:
          h0 = h.copy()
          h0.remove_spin()
          (ii,jj,ts) = extract.hopping_spinless(h0.intra)
  #        (ii,jj,ts) = extract.hopping_spinful(h.intra)
  else: (ii,jj,ts) = extract.hopping_spinless(h.intra)
  f = open(name,"w") # write file
  if mode=="abs": tso = np.abs(ts) 
  elif mode=="real": tso = ts.real 
  elif mode=="imag": tso = ts.imag
  else: raise
  for (i,j,t) in zip(ii,jj,tso):
    if skip(h.geometry.r[i],h.geometry.r[j]): continue
    if np.abs(t)<cutoff: continue
    if i==j: continue
    f.write(str(h.geometry.r[i][0])+"  ")
    f.write(str(h.geometry.r[i][1])+"  ")
    f.write(str(h.geometry.r[j][0])+"  ")
    f.write(str(h.geometry.r[j][1])+"  ")
    f.write(str(np.round(t,4))+"\n")
  f.close()



def mz(h,name="MZ.OUT"):
  if h.has_eh: raise
  if h.has_spin: ms = extract.mz(h.intra)
  else: raise
  np.savetxt(name,np.matrix([range(len(ms)),ms]).T)



def magnetization(h):
  """Write all the magnetizations"""
  if h.has_eh: raise
  if h.has_spin: 
    mx = extract.mx(h.intra)
    my = extract.my(h.intra)
    mz = extract.mz(h.intra)
  else: raise
  np.savetxt("MAGNETIZATION_X.OUT",np.matrix([h.geometry.x,h.geometry.y,mx]).T)
  np.savetxt("MAGNETIZATION_Y.OUT",np.matrix([h.geometry.x,h.geometry.y,my]).T)
  np.savetxt("MAGNETIZATION_Z.OUT",np.matrix([h.geometry.x,h.geometry.y,mz]).T)







# library to deal with the spectral properties of the hamiltonian
import numpy as np
import scipy.linalg as lg
import scipy.sparse.linalg as slg
import os
from numba import jit
from .. import filesystem as fs
from .. import parallel
from .. import interpolation


def get_qpi(h,reciprocal=True,nk=20,energies=np.linspace(-4.0,4.0,80),
        output_folder="MULTIQPI",nsuper=2,integrate=False,
        mode = "response",info=False,
        nunfold = 1, # flag for unfolding
        delta=1e-1,**kwargs):
    """Compute the QPI using a poor-mans convolution of the k-DOS"""
    if h.dimensionality!=2: raise
    if reciprocal: fR = h.geometry.get_k2K_generator() # get matrix
    else:  fR = lambda x: x # get identity
    qs0 = h.geometry.get_kmesh(nk=nk*nsuper,nsuper=nsuper)
    qs0 = qs0 - np.mean(qs0,axis=0)
    qs = np.array([fR(q) for q in qs0]) # convert
    if mode=="pm": # poor man mode
        from ..fermisurface import fermi_surface_generator
        es,ks,ds = fermi_surface_generator(h,reciprocal=False,info=info,
                energies=energies,delta=delta,
                full_bz=True, # flag for unfolding
                nsuper=nunfold,nk=nk,**kwargs)
        # this is a quick fix for unfolding, 
        # probably it should be better implemented
        ks = ks/nunfold # redefine the kpoints for unfolding
        # we now have the energies, k-points and DOS, lets do a convolution
        fp = lambda i: poor_man_qpi_single_energy(ks,ds[:,i],qs) # parallel function
        out = parallel.pcall(fp,range(len(es))) # compute in parallel
        dosa = np.sum(ds,axis=0) # array for the DOS
    ### alternative method ###
    elif mode=="response":
        from .epsilon import epsilonk
        out = epsilonk(h,energies=energies,nk=nk,delta=delta,qs=qs) # output
        es = energies # redefine the energies
        dosa = np.sum([o[1] for o in out],axis=1) # DOS
    else: raise
#    print(np.array(out).shape) ; exit()
    # now write everything #
    ########################################
    fs.rmdir(output_folder) # remove folder
    fs.mkdir(output_folder) # create folder
    kqpi = np.array([o[0] for o in out]).T # convert to array
    if integrate:
        kqpi = [np.mean(kqpi[:,0:i],axis=1) for i in range(len(es))]
        kqpi = [kp-np.min(kp) for kp in kqpi]
        kqpi = np.array(kqpi).T
    kdos = np.array([o[1] for o in out]).T # convert to array
    fo = open(output_folder+"/"+output_folder+".TXT","w")
    for i in range(len(es)): # loop over energies
        filename = output_folder+"_"+str(es[i])+"_.OUT" # name
        name = output_folder+"/"+filename
        np.savetxt(name,np.array([qs0[:,0],qs0[:,1],kqpi[:,i]]).T)
        np.savetxt(name+"_FS",np.array([qs0[:,0],qs0[:,1],kdos[:,i]]).T)
        fo.write(filename+"\n")
        name = output_folder+"/DOS.OUT"
    name = "DOS.OUT"
    np.savetxt(name,np.array([es,dosa]).T)
    fo.close()



def poor_man_qpi_single_energy(ks,ds,qs):
#    return poor_man_qpi_single_energy_brute_force(ks,ds,qs)
    return poor_man_qpi_convolve(ks,ds,qs)



def poor_man_qpi_convolve(ks,ds,qs):
    """Convolve the DOS to simmulate the QPI"""
    nq = int(np.sqrt(len(qs))) # number of qpoints
    nk = int(np.sqrt(len(ks))) # number of qpoints
    grid_kx, grid_ky = np.mgrid[0:1:nq*1j, 0:1:nq*1j] # kx and ky
    # DOS in a grid
    f = interpolation.interpolator2d(ks[:,0],ks[:,1],ds,mode="periodic")
    ksg = np.array([grid_kx, grid_ky]).reshape((2,nq*nq)).T # create points
    dsg = f(ksg).reshape((nq,nq)) # interpolate on a grid
    from ..convolution import selfconvolve
    out = selfconvolve(dsg).reshape((nq*nq)) # do the convolution
    out = interpolation.interpolator2d(ksg[:,0],ksg[:,1],out,mode="periodic")(qs[:,0:2])
    dsout = interpolation.interpolator2d(ksg[:,0],ksg[:,1],f(ksg),mode="periodic")(qs[:,0:2])
    print("Done QPI")
    return out,dsout


def poor_man_qpi_single_energy_brute_force(ks,ds,qs):
    """Do the convolution of the Fermi surfaces"""
    f0 = interpolation.interpolator2d(ks[:,0],ks[:,1],ds) # interpolated k-DOS
    def f(k):
        """Define a periodic function"""
        k = k[:,0:2]%1.
        o = f0(k)
        return o
    print("Doing")
    out = [np.mean(f(ks)*f(ks+q)) for q in qs]
    return np.array(out) # return output










from __future__ import print_function,division
import numpy as np
from .ldos import spatial_dos as spatial_density
from scipy.sparse import csc_matrix
import scipy.sparse.linalg as slg
import scipy.linalg as lg
from . import geometry
from . import timing
from . import klist

arpack_tol = 1e-06
arpack_maxiter = 10000


def diagonalize(intra,n=20,e=0.0,mode="arpack"):
  if mode=="arpack":
    eig,eigvec = slg.eigsh(csc_matrix(intra),k=n,which="LM",sigma=e,
                                  tol=arpack_tol,maxiter=arpack_maxiter)
  else:
    eig,eigvec = lg.eigh(csc_matrix(intra).todense())
  return eig,np.transpose(eigvec) # return


def restricted_density(intra,n=20,e=0.0,window=0.1,mode="arpack",
          window_mode="around"):
  (es,ws) = diagonalize(intra,n=n,e=e,mode=mode) # get energies and waves
  d = np.zeros(len(ws[0])) # initialize
  for (ie,iw) in zip(es,ws): # loop
    if window_mode=="around":
      if -window<(ie-e)<window: d = d + np.abs(iw)*np.abs(iw) # add
    elif window_mode=="below":
      if -window<(ie-e)<0.0: d = d + np.abs(iw)*np.abs(iw) # add
    elif window_mode=="above":
      if 0.0<(ie-e)<window: d = d + np.abs(iw)*np.abs(iw) # add
    elif window_mode=="filled":
      if ie<e: d = d + np.abs(iw)*np.abs(iw) # add
    else: raise
  return d # return contribution







def density(h,e=0.0,nk=20,mode="arpack",random=True,num_wf=20,
                window=0.1,nrep=3,name="DENSITY.OUT",
                window_mode="around",kpoints=None):
  """ Calculate the electronic density"""
  if h.intra.shape[0]<100: mode="full"
  if h.dimensionality==0: nk = 1 # single kpoint
  hkgen = h.get_hk_gen() # get generator
  d = np.zeros(h.intra.shape[0]) # initialize
  tr = timing.Testimator("DENSITY")
  from . import klist
  kpoints = klist.kmesh(h.dimensionality,nk=nk)
#  if kpoints is None: 
#    kpoints = [np.random.random(3) for ik in range(nk)]
  nk = len(kpoints) # given at input
  for ik in range(len(kpoints)):
    tr.remaining(ik,nk)
    k = kpoints[ik] # random k-point
    hk = csc_matrix(hkgen(k)) # get Hamiltonian
    d = d + restricted_density(hk,n=num_wf,e=e,window=window,
                   mode=mode,window_mode=window_mode) 
  d /= nk # normalize
  d = spatial_density(h,d) # sum if necessary
  geometry.write_profile(h.geometry,d,name=name,nrep=nrep)
  return d # return density












# routines to extract the local Hamiltonian
import numpy as np
from ..algebra import todense



def local_hamiltonian(h,m,i=0):
    """Given a certain Hamiltonian and a matrix,
    extract the local Hamiltonian"""
    m = todense(m) # dense array
    if not h.has_spin and not h.has_eh: # spinless and no e-h
        if i>=m.shape[0]: raise
        return np.array([[m[i,i]]]) # return a single number
    elif h.has_spin and not h.has_eh: # spinful and no e-h
        if i>=m.shape[0]//2: raise
        return m[2*i:2*i+2,2*i:2*i+2] # return a 2x2 block
    elif h.has_spin and h.has_eh: # with e-h
        if i>=m.shape[0]//4: raise
        return m[4*i:4*i+4,4*i:4*i+4] # return a 4x4 block
    else: raise





import os
import shutil
import subprocess
from pathlib import Path,PurePath


## this is just a wrapper to several functions to deal with files ##


def rmdir(a):
    """Remove a directory with its contents"""
    try: shutil.rmtree(a)
    except: pass

mkdir = os.makedirs # create a directory
cpdir = shutil.copytree
chdir = os.chdir

def execute(ll,background=True):
    if type(ll)!=list: ll = [ll] # convert to list
    subprocess.Popen(ll)

def joinpath(*args):
    return PurePath(*args)



from __future__ import print_function
import numpy as np
from scipy.sparse import csc_matrix,bmat
from numba import jit
from . import algebra


minimum_hopping = 1e-3


try: 
  from . import first_neighborsf90
  def find_first_neighbor(r1,r2):
      """Calls the fortran routine"""
      from . import first_neighborsf90 as fn
      r1t = np.matrix(r1).T
      r2t = np.matrix(r2).T
      nn = fn.number_neighborsf90(r1t,r2t)
      if nn==0: return []  # if no neighbors found
      pairs = np.array(fn.first_neighborsf90(r1t,r2t,nn))
      return pairs.T # return the pairs


except:
  def find_first_neighbor(r1,r2):
     """Calls the fortran routine"""
     r1 = np.array(r1)
     r2 = np.array(r2)
     nn = number_neighbors_jit(r1,r2) # number of first neighbors
     out = np.zeros((nn,2),dtype=np.int) # generate indexes
     out = find_first_neighbor_jit(r1,r2,out) # generate all the pairs
     return out

@jit(nopython=True)
def number_neighbors_jit(r1,r2):
    """Number of neighbors"""
    out = 0
    for i in range(len(r1)):
      for j in range(len(r2)):
         ri = r1[i]
         rj = r2[j]
         dr = ri-rj
         dr2 = dr[0]*dr[0] + dr[1]*dr[1] + dr[2]*dr[2]
         if 0.99<dr2<1.01: out += 1 # increase
    return out # number of neighbors

@jit(nopython=True)
def find_first_neighbor_jit(r1,r2,pairs):
    """Find the first neighbors"""
    out = 0
    for i in range(len(r1)):
      for j in range(len(r2)):
         ri = r1[i]
         rj = r2[j]
         dr = ri-rj
         dr2 = dr[0]*dr[0] + dr[1]*dr[1] + dr[2]*dr[2]
         if 0.99<dr2<1.01:
             pairs[out,0] = i
             pairs[out,1] = j
             out += 1 # increase
    return pairs # number of neighbors






def connections(r1,r2):
  """Return a list with the connections of each atom"""
  pairs = find_first_neighbor(r1,r2) # get the pairs of first neighbors
  out = [[] for i in range(len(r1))] # output list
  for p in pairs:
    out[int(p[0])].append(int(p[1])) 
  return out # return list





def parametric_hopping(r1,r2,fc,is_sparse=False):
  """ Generates a parametric hopping based on a function"""
  if is_sparse: # sparse matrix
    print("Sparse parametric hopping")
    m = np.matrix([[0.0j for i in range(len(r2))] for j in range(len(r1))])
    rows,cols,data = [],[],[]
    for i in range(len(r1)):
      for j in range(len(r2)):
        val = fc(r1[i],r2[j]) # add hopping based on function
        if abs(val) > minimum_hopping: # retain this hopping
         data.append(val)
         rows.append(i)
         cols.append(j)
    n = len(r2)
    m = csc_matrix((data,(rows,cols)),shape=(n,n))
  #  if not is_sparse: m = m.todense() # dense matrix
    return m
  else:
    n = len(r2)
    m = np.matrix(np.zeros((n,n),dtype=np.complex)) # complex matrix
    for i in range(len(r1)):
      for j in range(len(r2)):
        m[i,j] = fc(r1[i],r2[j])
    return m

 





def parametric_hopping_spinful(r1,r2,fc,is_sparse=False):
  """ Generates a parametric hopping based on a function, that returns
  a 2x2 matrix"""
  m = [[None for i in range(len(r2))] for j in range(len(r1))]
  for i in range(len(r1)):
    for j in range(len(r2)):
      val = fc(r1[i],r2[j]) # add hopping based on function
      m[i][j] = val # store this result
  m = bmat(m) # convert to matrix
  if not is_sparse: m = m.todense() # dense matrix
  return m







def generate_parametric_hopping(h,f=None,mgenerator=None,
             spinful_generator=False):
  """ Adds a parametric hopping to the hamiltonian based on an input function"""
  rs = h.geometry.r # positions
  g = h.geometry # geometry
  has_spin = h.has_spin # check if it has spin
  is_sparse = h.is_sparse
  if mgenerator is None: # no matrix generator given on input
    if f is None: raise # no function given on input
    if spinful_generator:
      raise
      print("WARNING, I am not sure why I programmed this")
      h.has_spin = True
      generator = parametric_hopping_spinful
    else:
      h.has_spin = False
      generator = parametric_hopping
    def mgenerator(r1,r2):
      return generator(r1,r2,f,is_sparse=is_sparse)
  else:
    if h.dimensionality==3: raise
  h.intra = mgenerator(rs,rs)
  if h.dimensionality == 0: pass
  elif h.dimensionality == 1:
    dr = g.a1
    h.inter = mgenerator(rs,rs+dr)
  elif h.dimensionality == 2:
    h.tx = mgenerator(rs,rs+g.a1)
    h.ty = mgenerator(rs,rs+g.a2)
    h.txy = mgenerator(rs,rs+g.a1+g.a2)
    h.txmy = mgenerator(rs,rs+g.a1-g.a2)
  elif h.dimensionality == 3:
    if spinful_generator: raise # not implemented
    h.is_multicell = True # multicell Hamiltonian
    from . import multicell
    multicell.parametric_hopping_hamiltonian(h,fc=f)
  else: raise
  # check that the sparse mde is set ok
  if is_sparse and type(h.intra)==type(np.matrix([[]])):
    h.is_sparse = False
    h.turn_sparse() # turn the matrix sparse
  if not is_sparse and type(h.intra)!=type(np.matrix([[]])):
    h.is_sparse = True
    h.turn_dense() # turn the matrix sparse
  if has_spin: # Hamiltonian should be spinful
    h.has_spin = False
    h.turn_spinful()
  return h




def neighbor_distances(g,n=4):
    """Return distances to neighbors"""
    nsuper = max([n//len(g.r)+3,3])
    g = g.supercell(nsuper) # create supercell
    r = g.r # positions
    n = len(r)
    out = np.zeros(n*n) # empty array
    out = neighbor_distances_jit(r,out) # distances
    out = np.round(out,6) # unique distances
    out = np.unique(out) # unique distances
    return np.array([out[i+1] for i in range(len(out)-1)]) # return


@jit(nopython=True)
def neighbor_distances_jit(r,out):
    n = len(r) # number of sites
    k = 0
    for i in range(n):
        for j in range(n):
            dr = r[i]-r[j]
            dis = dr[0]*dr[0]+dr[1]*dr[1]+dr[2]*dr[2]
            dis = np.sqrt(dis) # square root
            out[k] = dis # store
            k+=1 # increase
    return out



def neighbor_cells(num,dim=3):
  """Return indexes of neighboring cells,
  ordered from closer to further"""
  cells = [] # empty list
  if dim==0: return cells
  elif dim==1:
    for i in range(-num,num+1): cells.append([i,0,0])
  elif dim==2:
    for i in range(-num,num+1):
      for j in range(-num,num+1):
        cells.append([i,j,0])
  elif dim==3:
    for i in range(-num,num+1):
      for j in range(-num,num+1):
        for k in range(-num,num+1):
          cells.append([i,j,k])
  # now order the cells
  dis = [np.array(a).dot(np.array(a)) for a in cells] # distances
  cells = [y for (x,y) in zip(dis,cells)] # sort
  return cells # return the indexes



def neighbor_directions(g,cutoff=3):
    """Return the vectors pointing to neighbors"""
    dirs = []
    if g.dimensionality==0: return [[0.,0.,0.]] # zero dimensional
    elif g.dimensionality==1: # one dimensional
      for i1 in range(-cutoff,cutoff+1): dirs.append([i1,0,0])
    elif g.dimensionality==2: # two dimensional
      for i1 in range(-cutoff,cutoff+1):
        for i2 in range(-cutoff,cutoff+1):
          dirs.append([i1,i2,0])
    elif g.dimensionality==3: # three dimensional
      for i1 in range(-cutoff,cutoff+1):
        for i2 in range(-cutoff,cutoff+1):
          for i3 in range(-cutoff,cutoff+1):
            dirs.append([i1,i2,i3])
    else: raise # not implemented
    dirs = [np.array(d) for d in dirs]
    return dirs # return directions




# library to calculate topological properties
from __future__ import print_function
import numpy as np
from scipy.sparse import bmat, csc_matrix
import scipy.linalg as lg
import scipy.sparse.linalg as slg
from . import multicell
from . import klist
from . import operators
from . import inout
from . import timing
from . import algebra
from . import parallel


arpack_tol = 1e-5
arpack_maxiter = 10000


def write_berry(h,kpath=None,dk=0.01,window=None,max_waves=None,nk=600,
      mode="Wilson",delta=0.001,reciprocal=False,operator=None,
      silent = True):
  """Calculate and write in file the Berry curvature"""
  operator = get_operator(h,operator)
  if kpath is None: kpath = klist.default(h.geometry,nk=nk) # take default kpath
  tr = timing.Testimator("BERRY CURVATURE",silent=silent)
  ik = 0
  if operator is not None: mode="Green" # Green function mode
  def getb(k):
    if reciprocal:  k = h.geometry.get_k2K_generator()(k) # convert
    if mode=="Wilson":
      b = berry_curvature(h,k,dk=dk,window=window,max_waves=max_waves)
    if mode=="Green":
      f = h.get_gk_gen(delta=delta) # get generator
      b = berry_green(f,k=k,operator=operator) 
    return str(k[0])+"   "+str(k[1])+"   "+str(b)+"\n"
  fo = open("BERRY_CURVATURE.OUT","w") # open file
  if parallel.cores==1: # serial execution
    for k in kpath:
      tr.remaining(ik,len(kpath))
      ik += 1
      fo.write(getb(k)) # write result
      fo.flush()
  else: # parallel execution
      out = parallel.pcall(getb,kpath)
      for o in out: fo.write(o) # write
  fo.close() # close file
  m = np.genfromtxt("BERRY_CURVATURE.OUT").transpose()
  return np.array(range(len(m[0]))),m[2]










def berry_phase(h,nk=20,kpath=None):
  """ Calculates the Berry phase of a Hamiltonian"""
  if h.dimensionality==0: raise
  elif h.dimensionality == 1:
    ks = np.linspace(0.,1.,nk,endpoint=False) # list of kpoints
  elif h.dimensionality > 1: # you must provide a kpath
      if kpath is None: 
          print("You must provide a k-path")
          raise # error
      ks = kpath # continue
      nk = len(kpath) # redefine
  else: raise # otherwise
  hkgen = h.get_hk_gen() # get Hamiltonian generator
  wf0 = occupied_states(hkgen,ks[0]) # get occupied states, first k-point
  wfold = wf0.copy() # copy
  m = np.matrix(np.identity(len(wf0))) # initialize as the identity matrix
  for ik in range(1,len(ks)): # loop over k-points, except first one
    wf = occupied_states(hkgen,ks[ik])  # get waves
    m = m@uij(wfold,wf)   # get the uij   and multiply
    wfold = wf.copy() # this is the new old
  m = m@uij(wfold,wf0)   # last one
  d = lg.det(m) # calculate determinant
  phi = np.arctan2(d.imag,d.real)
  open("BERRY_PHASE.OUT","w").write(str(phi/np.pi)+"\n")
  return phi # return Berry phase








def berry_curvature(h,k,dk=0.01,window=None,max_waves=None):
  """ Calculates the Berry curvature of a 2d hamiltonian"""
  if h.dimensionality != 2: raise # only for 2d
  k = np.array([k[0],k[1]]) 
  dx = np.array([dk,0.])
  dy = np.array([0.,dk])
# get the function that returns the occ states
  occf = occ_states_generator(h,k,window=window,max_waves=max_waves)  
  # get the waves
#  print("Doing k-point",k)
  wf1 = occf(k-dx-dy) 
  wf2 = occf(k+dx-dy) 
  wf3 = occf(k+dx+dy) 
  wf4 = occf(k-dx+dy) 
  dims = [len(wf1),len(wf2),len(wf3),len(wf4)] # number of vectors
  if max(dims)!=min(dims): # check that the dimensions are fine 
#    print("WARNING, skipping this k-point",k)
    return 0.0 # if different number of vectors
  # get the uij  
  m = uij(wf1,wf2)@uij(wf2,wf3)@uij(wf3,wf4)@uij(wf4,wf1)
  d = lg.det(m) # calculate determinant
  phi = np.arctan2(d.imag,d.real)/(4.*dk*dk)
  return phi


def occ_states_generator(h,k,window=None,max_waves=None):
  """Return a function that generates the occupied wavefunctions"""
  hk_gen = h.get_hk_gen() # get hamiltonian generator
# no need of h anymore
  return lambda k: occupied_states(hk_gen,k,window=window,max_waves=max_waves) 



def occ_states2d(h,k):
  """Input is a Hamiltonian"""
  hk_gen = h.get_hk_gen() # get hamiltonian generator
  return occupied_states(hk_gen,k)


def occupied_states(hkgen,k,window=None,max_waves=None):
  """ Returns the WF of the occupied states in a 2d hamiltonian"""
  hk = hkgen(k) # get hamiltonian
  if max_waves is None: es,wfs = algebra.eigh(hk) # diagonalize all waves
  else:  es,wfs = slg.eigsh(csc_matrix(hk),k=max_waves,which="SA",
                      sigma=0.0,tol=arpack_tol,maxiter=arpack_maxiter)
  wfs = np.conjugate(wfs.transpose()) # wavefunctions
  occwf = []
  for (ie,iw) in zip(es,wfs):  # loop over states
    if window is None: # no energy window
      if ie < 0:  # if below fermi
        occwf.append(iw)  # add to the list
    else: # energy window provided
      if -abs(window)< ie < 0:  # between energy window and fermi
        occwf.append(iw)  # add to the list
  return np.array(occwf)




def uij(wf1,wf2):
  """ Calcultes the matrix product of two sets of input wavefunctions"""
  out =  np.matrix(np.conjugate(wf1))@(np.matrix(wf2).T)  # faster way
  return out


def uij_slow(wf1,wf2):
  m = np.matrix(np.zeros((len(wf1),len(wf2)),dtype=np.complex))
  for i in range(len(wf1)):
    for j in range(len(wf2)):
      m[i,j] = np.conjugate(wf1[i]).dot(wf2[j])
  return m


def precise_chern(h,dk=0.01, mode="Wilson",delta=0.0001,operator=None):
  """ Calculates the chern number of a 2d system """
  from scipy import integrate
  err = {"epsabs" : 1.0, "epsrel": 1.0,"limit" : 10}
#  err = [err,err]
  def f(x,y): # function to integrate
    if mode=="Wilson":
      return berry_curvature(h,np.array([x,y]),dk=dk)
    if mode=="Green":
       f2 = h.get_gk_gen(delta=delta) # get generator
       return berry_green(f2,k=[x,y,0.],operator=operator) 
  c = integrate.dblquad(f,0.,1.,lambda x : 0., lambda x: 1.,epsabs=0.01,
                          epsrel=0.01)
  chern = c[0]/(2.*np.pi)
  open("CHERN.OUT","w").write(str(chern)+"\n")
  return chern


def hall_conductivity(h,dk=-1,n=1000):
  c = 0.0 
  nk = int(np.sqrt(n)) # estimate
  if dk<0: dk = 1./float(2*nk) # automatic dk
  for i in range(n):
    k = np.random.random(2) # random kpoint
    c += berry_curvature(h,k,dk=dk)
  c = c/(2*np.pi*n) # normalize
  return c




def mesh_chern(h,dk=-1,nk=10,delta=0.0001,mode="Wilson",operator=None):
  """ Calculates the chern number of a 2d system """
  c = 0.0
  ks = [] # array for kpoints
  bs = [] # array for berrys
  if dk<0: dk = 1./float(2*nk) # automatic dk
  if operator is not None and mode=="Wilson":
    print("Switching to Green mode in topology")
    mode="Green"
  # create the function
  def fberry(k): # function to integrate
    if mode=="Wilson":
      return berry_curvature(h,k,dk=dk)
    if mode=="Green":
       f2 = h.get_gk_gen(delta=delta) # get generator
       return berry_green(f2,k=[k[0],k[1],0.],operator=operator) 
  ##################
  for x in np.linspace(0.,1.,nk,endpoint=False):
    for y in np.linspace(0.,1.,nk,endpoint=False):
      ks.append([x,y]) # create kpoints
#  tr = timing.Testimator("CHERN NUMBER")
  ik = 0
  bs = parallel.pcall(fberry,ks) # compute all the Berry curvatures
#  for k in ks: # loop
#    tr.remaining(ik,len(ks))
#    ik += 1 # increase
#    b = fberry(k) # get berry curvature
#    bs.append(b)
  # write in file
  fo = open("BERRY_CURVATURE.OUT","w") # open file
  for (k,b) in zip(ks,bs):
    fo.write(str(k[0])+"   ")
    fo.write(str(k[1])+"   ")
    fo.write(str(b)+"\n")
  fo.close() # close file
  ################
  c = np.sum(bs) # sum berry curvatures
  c = c/(2.*np.pi*nk*nk)
  open("CHERN.OUT","w").write(str(c)+"\n")
  return c



def berry_map(h,dk=-1,nk=40,reciprocal=True,nsuper=1,window=None,
               max_waves=None,mode="Wilson",delta=0.001,operator=None):
  """ Calculates the chern number of a 2d system """
  if operator is not None: mode="Green" # Green function mode
  c = 0.0
  ks = [] # array for kpoints
  if dk<0: dk = 5./float(2*nk) # automatic dk
  if reciprocal: R = h.geometry.get_k2K()
  else: R = np.matrix(np.identity(3))
  fo = open("BERRY_MAP.OUT","w") # open file
  nt = nk*nk # total number of points
  ik = 0
  ks = [] # list with kpoints
  from . import parallel
  for x in np.linspace(-nsuper,nsuper,nk,endpoint=False):
    for y in np.linspace(-nsuper,nsuper,nk,endpoint=False):
        ks.append([x,y,0.])
  tr = timing.Testimator("BERRY CURVATURE",maxite=len(ks))
  def fp(ki): # function to compute the Berry curvature
      if parallel.cores == 1: tr.iterate()
      else: print("Doing",ki)
      r = np.matrix(ki).T # real space vectors
      k = np.array((R*r).T)[0] # change of basis
      if mode=="Wilson":
         b = berry_curvature(h,k,dk=dk,window=window,max_waves=max_waves)
      if mode=="Green":
         f = h.get_gk_gen(delta=delta) # get generator
         b = berry_green(f,k=k,operator=operator) 
      return b
  bs = parallel.pcall(fp,ks) # compute all the Berry curvatures
  for (b,k) in zip(bs,ks): # write everything
      fo.write(str(k[0])+"   "+str(k[1])+"     "+str(b)+"\n")
      fo.flush()
  fo.close() # close file





def smooth_gauge(w1,w2):
  """Perform a gauge rotation so that the second set of waves are smooth
  with respect to the first one"""
  m = uij(w1,w2) # matrix of wavefunctions
  U, s, V = np.linalg.svd(m, full_matrices=True) # sing val decomp
  R = (U@V).H # rotation matrix
  wnew = w2.copy()*0j # initialize
  wold = w2.copy() # old waves
  for ii in range(R.shape[0]):
    for jj in range(R.shape[0]):
      wnew[ii] += R[jj,ii]*wold[jj]
  return wnew




def z2_vanderbilt(h,nk=30,nt=100,nocc=None,full=False):
  """ Calculate Z2 invariant according to Vanderbilt algorithm"""
  out = [] # output list
  path = np.linspace(0.,1.,nk) # set of kpoints
  fo = open("WANNIER_CENTERS.OUT","w")
  if full:  ts = np.linspace(0.,1.0,nt,endpoint=False)
  else:  ts = np.linspace(0.,0.5,nt,endpoint=False)
  wfall = [[occ_states2d(h,np.array([k,t,0.,])) for k in path] for t in ts] 
  # select a continuos gauge for the first wave
  for it in range(len(ts)-1): # loop over ts
    wfall[it+1][0] = smooth_gauge(wfall[it][0],wfall[it+1][0]) 
  for it in range(len(ts)): # loop over t points
    row = [] # empty list for this row
    t = ts[it] # select the t point
    wfs = wfall[it] # get set of waves 
    for i in range(len(wfs)-1):
      wfs[i+1] = smooth_gauge(wfs[i],wfs[i+1]) # transform into a smooth gauge
#      m = uij(wfs[i],wfs[i+1]) # matrix of wavefunctions
    m = uij(wfs[0],wfs[len(wfs)-1]) # matrix of wavefunctions
    evals = lg.eigvals(m) # eigenvalues of the rotation 
    x = np.angle(evals) # phase of the eigenvalues
    fo.write(str(t)+"    ") # write pumping variable
    row.append(t) # store
    for ix in x: # loop over phases
      fo.write(str(ix)+"  ")
      row.append(ix) # store
    fo.write("\n")
    out.append(row) # store
  fo.close()
  return np.array(out).transpose() # transpose the map


def z2_invariant(h,nk=20,nt=20,nocc=None):
  """Compute Z2 invariant with pumping of Wannier centers"""
  return wannier_winding(h,nk=nk,nt=nt,nocc=nocc,full=False) 




def chern(h,**kwargs):
  """Compute Chern invariant"""
  return mesh_chern(h,**kwargs) # workaround
  # the wannier winding does not work
  c = wannier_winding(h,full=True,**kwargs) 
  open("CHERN.OUT","w").write(str(c))
  return c




def wannier_winding(h,nk=40,nt=40,nocc=None,full=True):
  m = z2_vanderbilt(h,nk=nk,nt=nt,nocc=nocc,full=full)
  x = m[0]
  # find the position of the maximum gap at every t
  fermis = x*0. # maximum gap
  for it in range(len(x)): # loop over times
    imax,jmax = None,None
    dmax = -1 # initialize
    gapangle = None # initialize
    maxgap = -1.0 # maximum gap
    gapangle = ((m[1][it]+0.5)%1)*np.pi # initialize
    for i in range(1,len(m)):
      for j in range(i+1,len(m)):
        for ipi in [0.,1.]:
          ip = np.exp(1j*m[i][it]) # center of wave i
          jp = np.exp(1j*m[j][it]) # center of wave j
          angle = np.angle(ip+jp)+np.pi*ipi # get the angle
          dp = np.exp(1j*angle) # now obtain this middle point gap
          mindis = 4.0 # calculate minimum distance
          for k in range(1,len(m)): # loop over centers
            kp = np.exp(1j*m[k][it]) # center of wave k
            dis = np.abs(dp-kp) # distance between the two points 
            if dis<mindis: mindis = dis+0. # update minimum distance
          if mindis>maxgap: # if found a bigger gap
            maxgap = mindis+0. # maximum distance
            gapangle = np.angle(dp) # update of found bigger gap
    fermis[it] = gapangle
  # now check the number of cuts of each wannier center
  def angleg(a,b,c):
    """Function to say if a jump has been made or not"""
    d = np.sin(a-b) + np.sin(b-c) + np.sin(c-a)
    return -d

  if full: # for the Chern number
    raise
    # this part is wrong
    #####################
    cuts = 0 # start with 0
#    print(fermis)
    for i in range(1,len(m)): # loop over waves 
      cwf = m[i] # center of the wave
      for it in range(len(x)-1): # loop over times
        s1 = np.sign(fermis[it]-cwf[it])
        s2 = np.sign(fermis[it+1]-cwf[it])
#        print(s1,s2)
        cuts += (s1-s2)/2. 
    return cuts

  else: # for the Z2 invariant
    parity = 1 # start with
    for i in range(1,len(m)): # loop over waves 
      cwf = m[i] # center of the wave
      for it in range(len(x)-1): # loop over times
        s = np.sign(angleg(fermis[it],fermis[it+1],cwf[it])) 
        if s<0.:  parity *= -1 # add a minus sign
    return parity










def operator_berry(hin,k=[0.,0.],operator=None,delta=0.00001,ewindow=None):
  """Calculates the Berry curvature using an arbitrary operator"""
  h = multicell.turn_multicell(hin) # turn to multicell form
  dhdx = multicell.derivative(h,k,order=[1,0]) # derivative
  dhdy = multicell.derivative(h,k,order=[0,1]) # derivative
  hkgen = h.get_hk_gen() # get generator
  hk = hkgen(k) # get hamiltonian
  (es,ws) = algebra.eigh(hkgen(k)) # initial waves
  ws = np.conjugate(np.transpose(ws)) # transpose the waves
  n = len(es) # number of energies
  from .berry_curvaturef90 import berry_curvature as bc90
  if operator is None: operator = np.identity(dhdx.shape[0],dtype=np.complex)
  b = bc90(dhdx,dhdy,ws,es,operator,delta) # berry curvature
  return b*np.pi*np.pi*8 # normalize so the sum is 2pi Chern



def operator_berry_bands(hin,k=[0.,0.],operator=None,delta=0.00001):
  """Calculates the Berry curvature using an arbitrary operator"""
  h = multicell.turn_multicell(hin) # turn to multicell form
  dhdx = multicell.derivative(h,k,order=[1,0]) # derivative
  dhdy = multicell.derivative(h,k,order=[0,1]) # derivative
  hkgen = h.get_hk_gen() # get generator
  hk = hkgen(k) # get hamiltonian
  (es,ws) = algebra.eigh(hkgen(k)) # initial waves
  ws = np.conjugate(np.transpose(ws)) # transpose the waves
  from .berry_curvaturef90 import berry_curvature_bands as bcb90
  if operator is None: operator = np.identity(dhdx.shape[0],dtype=np.complex)
  bs = bcb90(dhdx,dhdy,ws,es,operator,delta) # berry curvatures
  return (es,bs*np.pi*np.pi*8) # normalize so the sum is 2pi Chern







def spin_chern(h,nk=40,delta=0.00001,k0=[0.,0.],expandk=1.0):
  """Calculate the spin Chern number"""
  kxs = np.linspace(-.5,.5,nk,endpoint=False)*expandk + k0[0]
  kys = np.linspace(-.5,.5,nk,endpoint=False)*expandk + k0[1]
  kk = [] # list of kpoints
  for i in kxs:
    for j in kys:
      kk.append(np.array([i,j])) # store vector
  sz = operators.get_sz(h) # get sz operator
  bs = [operator_berry(h,k=ki,operator=sz,delta=delta) for ki in kk] # get all berries
  fo = open("BERRY_CURVATURE_SZ.OUT","w") # open file
  for (k,b) in zip(kk,bs):
    fo.write(str(k[0])+"   ")
    fo.write(str(k[1])+"   ")
    fo.write(str(b)+"\n")
  fo.close() # close file
  bs = np.array(bs)/(2.*np.pi) # normalize by 2 pi
  return sum(bs)/len(kk)


def write_spin_berry(h,kpath,delta=0.00001,operator=None):
  """Calculate and write in file the Berry curvature"""
  if operator is None: sz = operators.get_sz(h) # get sz operator
  else: sz = operator # assign operator
  be = [operator_berry(h,k=k,operator=sz,delta=delta) for k in kpath] 
  fo = open("BERRY_CURVATURE_SZ.OUT","w") # open file
  for (k,b) in zip(kpath,be):
    fo.write(str(k[0])+"   ")
    fo.write(str(k[1])+"   ")
    fo.write(str(b)+"\n")
  fo.close() # close file






def precise_spin_chern(h,delta=0.00001,tol=0.1):
  """ Calculates the chern number of a 2d system """
  from scipy import integrate
  err = {"epsabs" : 0.01, "epsrel": 0.01,"limit" : 20}
  sz = operators.get_sz(h) # get sz operator
  def f(x,y): # function to integrate
    return operator_berry(h,np.array([x,y]),delta=delta,operator=sz)
  c = integrate.dblquad(f,0.,1.,lambda x : 0., lambda x: 1.,epsabs=tol,
                          epsrel=tol)
  return c[0]/(2.*np.pi)




def berry_green_generator(f,k=[0.,0.,0.],dk=0.05,operator=None,
              full=False):
  """Function that returns the energy resolved Berry curvature"""
  k = np.array(k) # set as array
  dx = np.array([dk,0.,0.])
  dy = np.array([0.,dk,0.])
  def fint(e): # function to integrate
#    g = f(e=e,k=k) # compute at this k and this energy
    gxp = f(e=e,k=k+dx) # compute at this k and this energy
    gxm = f(e=e,k=k-dx) # compute at this k and this energy
    gyp = f(e=e,k=k+dy) # compute at this k and this energy
    gym = f(e=e,k=k-dy) # compute at this k and this energy
    g = (gxp + gyp + gxm + gym)/4. # average Green function
    # Now apply the formula
    gI = algebra.inv(g) # inverse
    omega = ((gxp-gxm)@(gyp-gym) - (gyp-gym)@(gxp-gxm))@gI
#    omega = g*((gxp.I-gxm.I)*(gyp-gym) -(gyp.I-gym.I)*(gxp-gxm))
#    omega += -g*(gyp.I-gym.I)*(gxp-gxm)
    if operator is not None: omega = operator(omega,k=k) 
    if full: return omega/(4.*dk*dk*2.*np.pi) # return the full matrix
    else: return np.trace(omega)/(4.*dk*dk*2.*np.pi) # return contribution
  return fint # return the function




def berry_green(f,emin=-10.0,k=[0.,0.,0.],ne=100,dk=0.0001,operator=None):
  """Return the Berry curvature using Green functions"""
  import scipy.integrate as integrate
  fint = berry_green_generator(f,k=k,dk=dk,operator=operator) 
  es = np.linspace(emin,0.,ne) # energies used for the integration
  ### The original function is defined in the coplex plane,
  # we will do a change of variables of the form z = re^(iphi) - r0
  # so that dz = re^(iphi) i dphi
  def fint2(x):
    """Function to integrate using a complex contour, from 0 to 1"""
    z0 = emin*np.exp(-1j*x*np.pi)/2.
    z = z0 + emin/2.
    return -(fint(z)*z0).imag*np.pi # integral after the change of variables
  return integrate.quad(fint2,0.0,1.0,limit=60,epsabs=0.1,epsrel=0.1)[0]
#  return integrate.quad(fint,emin,0.0,limit=60,epsabs=0.01,epsrel=0.01)[0]
#  return np.sum([fint(e) for e in es]) # return


def berry_operator(h,delta=1e-1,**kwargs):
    """Return ap operator that computes the Berry curvature for a certain
    wavefunction"""
    if h.dimensionality!=2: raise
    h = h.copy()
    h.turn_dense()
    hk = h.get_hk_gen() # get generator
    gk = h.get_gk_gen(delta=delta) # get generator
    def bk(k): return berry_green_generator(gk,k=k,**kwargs) 
    def outf(w,k=[0.,0.,0.]):
        m = hk(k) # bloch Hamiltonian
        e = algebra.braket_wAw(w,m) # energy
        o = bk(k)(e)*delta # Berry curvature
        return o*w # return a vector
    return outf


def berry_green_map_kpoint(h,emin=None,k=[0.,0.,0.],
        ne=100,dk=0.0001,operator=None,integral_mode="complex",
                  delta=0.002,integral=True,eps=1e-1,
                  energy=0.0,emax=0.0):
  """Return the Berry curvature map at a certain kpoint"""
  f = h.get_gk_gen(delta=delta,canonical_phase=True) # green function generator
  fgreen = berry_green_generator(f,k=k,dk=dk,operator=operator,full=True) 
  # No minimum energy provided
  if emin is None and integral:
      emin = algebra.eigvalsh(h.get_hk_gen()(k))[0] - 1.0
      print("Minimum energy",emin)
  def fint(x):  
#    return fgreen(x).trace()[0,0] # return diagonal
    return np.diag(fgreen(x)) # return diagonal
  ### The original function is defined in the complex plane,
  # we will do a change of variables of the form z = re^(iphi) - r0
  # so that dz = re^(iphi) i dphi
  if integral: # integrate up to the fermi energy
    es = np.linspace(emin,0.,ne) # energies used for the integration
    def fint2(x):
      """Function to integrate using a complex contour, from 0 to 1"""
      de = emax-emin # energy window of the integration
      ce = de/2. # center of the circle
      z0 = -ce*np.exp(-1j*x*np.pi) # parametrize the circle
      z = z0 + (emin+emax)/2. # shift the circle
      print("Evaluating",x)
      return -(fint(z)*z0).imag*np.pi # integral after the change of variables
    from .integration import integrate_matrix
    out = integrate_matrix(fint2,xlim=[0.,1.],eps=eps)
    out = out.real # turn real
  else: # evaluate at the fermi energy
    out = fint(energy).real
  return out # return result


def spatial_berry_density(h,**kwargs):
    return berry_green_map(h,integral=False,**kwargs)



def berry_green_map(h,nrep=5,k=[0.,0.,0.],operator=None,nk=None,**kwargs):
  """
  Write the Berry curvature of a kpoint in a file
  """
  if operator is not None: # this is a dirty workaround
    if type(operator) is str:
      operator = h.get_operator(operator) 
    else: pass
  if nk is None: # kpoint given
    out = berry_green_map_kpoint(h,k=k,operator=operator,**kwargs) 
  else: # kpoint not given
    if operator is not None: 
        from . import gauge
        operator = gauge.Operator2canonical_gauge(h,operator)
        print("Fixing the gauge in the operator")
    ks = klist.kmesh(h.dimensionality,nk=nk) # kpoints
    def f(ki):
        print("kpoint",ki)
        return berry_green_map_kpoint(h,k=ki,operator=operator,**kwargs)
    out = parallel.pcall(f,ks) # compute all
    out = np.mean(out,axis=0) # resum
  from . import geometry
  from .ldos import spatial_dos
  # write in a file
  geometry.write_profile(h.geometry,
          spatial_dos(h,out),name="BERRY_MAP.OUT",nrep=nrep)
  return out



def berry_density(h,k=[0.,0.,0.],operator=None,delta=0.02,dk=0.02):
  """Compute the Berry density"""
  f = h.get_gk_gen(delta=delta,canonical_phase=True) # green function generator
  fgreen = berry_green_generator(f,k=k,dk=dk,operator=operator,full=False) 
  return fgreen(0.0).real


def berry_density_map(h,nk=40,reciprocal=True,nsuper=1,
               delta=None,operator=None,dk=0.01):
  """Compute a Berry density map"""
  if delta is None: delta = 5./nk
  if reciprocal: R = h.geometry.get_k2K()
  else: R = np.matrix(np.identity(3))
  fo = open("BERRY_DENSITY_MAP.OUT","w") # open file
  nt = nk*nk # total number of points
  ik = 0
  ks = [] # list with kpoints
  for x in np.linspace(-nsuper,nsuper,nk,endpoint=False):
    for y in np.linspace(-nsuper,nsuper,nk,endpoint=False):
        ks.append([x,y,0.])
  tr = timing.Testimator("BERRY DENSITY",maxite=len(ks))
  def fp(ki): # function to compute the Berry curvature
      if parallel.cores == 1: tr.iterate()
      else: print("Doing",ki)
      r = np.matrix(ki).T # real space vectors
      k = np.array((R*r).T)[0] # change of basis
      b = berry_density(h,k=k,operator=operator,dk=dk) # get the density
      return b
  bs = parallel.pcall(fp,ks) # compute all the Berry curvatures
  for (b,k) in zip(bs,ks): # write everything
      fo.write(str(k[0])+"   "+str(k[1])+"     "+str(b)+"\n")
      fo.flush()
  fo.close() # close file



def chern_density(h,nk=10,operator=None,delta=0.02,dk=0.02,
        es=np.linspace(-1.0,1.0,40)):
  """Compute the Chern density as a function of the energy"""
  ks = klist.kmesh(h.dimensionality,nk=nk)
  cs = np.zeros(es.shape[0]) # initialize
  f = h.get_gk_gen(delta=delta,canonical_phase=True) # green function generator
  tr = timing.Testimator("CHERN DENSITY",maxite=len(ks))
  from . import parallel
  def fp(k): # compute berry curvatures
    if parallel.cores==1: tr.iterate()
    else: print(k)
#    k = np.random.random(3)
    fgreen = berry_green_generator(f,k=k,dk=dk,operator=operator,full=False) 
    return np.array([fgreen(e).real for e in es])
  out = parallel.pcall(fp,ks) # compute everything
  for o in out: cs += o # add contributions
  cs = cs/(len(ks)*np.pi*2) # normalize
  from scipy.integrate import cumtrapz
  csi = cumtrapz(cs,x=es,initial=0) # integrate
  np.savetxt("CHERN_DENSITY.OUT",np.matrix([es,cs]).T)
  np.savetxt("CHERN_DENSITY_INTEGRATED.OUT",np.matrix([es,csi]).T)








hall_conductivity = chern




def get_operator(h,op):
    """ Wrapper for operators """
    if op is None: return None
    if type(op)==str: # string
        if op=="valley": return h.get_operator("valley",projector=True) 
        else: return h.get_operator("valley",projector=True) 
    if callable(op): return op # function
    if type(op)==np.array: return op



from .topologytk import realspace

real_space_chern = realspace.real_space_chern




from __future__ import print_function
import numpy as np


def ylm2xyz_l1():
  """Return the matrix that converts the cartesian into spherical harmonics"""
  m = np.matrix([[0.0j for i in range(3)] for j in range(3)])
  s2 = np.sqrt(2.)
  m[1,2] = 1. # pz
  m[0,0] = 1./s2 # dxz
  m[2,0] = -1./s2 # dxz
  m[0,1] = 1j/s2 # dyz
  m[2,1] = 1j/s2 # dyz
  return m


def ylm2xyz_l2():
  """Return the matrix that converts the cartesian into spherical harmonics"""
  m = np.matrix([[0.0j for i in range(5)] for j in range(5)])
  s2 = np.sqrt(2.)
  m[2,0] = 1. # dz2
  m[1,1] = 1./s2 # dxz
  m[3,1] = -1./s2 # dxz
  m[1,2] = 1j/s2 # dyz
  m[3,2] = 1j/s2 # dyz
  m[0,3] = 1j/s2 # dxy
  m[4,3] = -1j/s2 # dxy
  m[0,4] = 1./s2 # dx2y2
  m[4,4] = 1./s2 # dx2y2
  return m # return change of bassi matrix


# names of orbitals
dorbs = ["dz2","dxz","dyz","dxy","dx2-y2"]
porbs = ["px","py","pz"]
sorbs = ["s"]

def names_soc_orbitals(specie):
  """Names of the orbitals, order coherent with angular change of basis"""
  name = get_element(specie) # get the true name of the atom
  datoms = ["Sc","Ti","V","Cr","Mn","Fe","Co","Ni","Cu","Zn"]
  datoms += ["Y","Zr","Nb","Mo","Tc","Ru","Rh","Pd","Ag","Cd"]
  datoms += ["Hf","Ta","W","Re","Os","Ir","Pt","Au","Hg"]
  patoms = ["B","C","N","O","F","P"]
  patoms += ["Al","Si","P","S","Cl"]
  patoms += ["Ga","Ge","As","Se","Br"]
  patoms += ["In","Sn","An","Te","I"]
  patoms += ["Tl","Pb","Bi","Po","At"]
  if name in datoms: return dorbs # for dorbitals
  elif name in patoms: return porbs # for dorbitals
  else: raise



def get_element(name):
  """Gets the true name of an element, assumes that there might be a number"""
  out = "" # initialize
  if name[-1] in [str(i) for i in range(10)]:
    for i in range(len(name)-1): # except the last one
      out += name[i] # add to the string
    return get_element(out) # recall the function, just in case two digits
  return name




def angular_momentum(orbs):
  """Return the angular momentum operator in a certain basis
  of cartesian operators"""
  for o in orbs:
    if o in sorbs: # check if they are s orbitals
      l=0
      orbnames = sorbs
      break
  for o in orbs:
    if o in porbs: # check if they are s orbitals
      l=1
      orbnames = porbs
      break
  for o in orbs:
    if o in dorbs: # check if they are s orbitals
      l=2
      orbnames = dorbs
      break
  nm = 2*l + 1 # number of components
  zero = np.matrix([[0.0j for i in range(nm)] for j in range(nm)])
  # initialize matrices
  lz = zero.copy()
  lx = zero.copy()
  ly = zero.copy()
  lm = zero.copy()
  lp = zero.copy()
  # create l+ and l- and lz
  for m in range(-l,l): # loop over m components
    val = np.sqrt((l-m)*(l+m+1)) # value of the cupling
    im = m + l
    lp[(im+1),im] = val # up channel
  for m in range(-l,l+1): # loop over m components
    im = m + l
    lz[im,im] = m # value of lz, up channel
  lm = lp.H # adjoint
  lx = (lp + lm) /2.
  ly = -1j*(lp - lm) /2.
  # at this point lx,ly,lz are written in the basis of spherical harmonics
  if l==2: R = ylm2xyz_l2() # get change of basis matrix
  if l==1: R = ylm2xyz_l1() # get change of basis matrix
  lx = R.H * lx * R # change to cartesian orbitals
  ly = R.H * ly * R # change to cartesian orbitals
  lz = R.H * lz * R # change to cartesian orbitals
  
  # at this point the angular operators are written in the cartesian basis
  # now you must project onto the input manifold
  proj = np.matrix([[0.0j for i in range(nm)] for j in range(len(orbs))])
  for i in range(len(orbs)): # loop over input cartesian orbitals
    for j in range(len(orbnames)): # loop over full cartesian orbitals
      if orbs[i] == orbnames[j]: # if same orbital
        proj[i,j] = 1. # identity
#        print(orbs[i],orbnames[j],i,j) 
        break
#      print(orbs,orbnames,i,j)
#      raise # raise error if this point is reached
#  print(proj)
#  raise
  proj = proj.H
  # now project
  lx = proj.H * lx * proj # change to cartesian orbitals
  ly = proj.H * ly * proj # change to cartesian orbitals
  lz = proj.H * lz * proj # change to cartesian orbitals
  return (lx,ly,lz) # return the three matrices







import numpy as np
import os
from scipy.sparse import csr_matrix
import pickle
from . import filesystem as fs

def save_sparse_csr(filename,array):
    array = csr_matrix(array)
    np.savez(filename,data = array.data ,indices=array.indices,
             indptr =array.indptr, shape=array.shape )

def load_sparse_csr(filename,is_sparse=True):
    loader = np.load(filename)
    m = csr_matrix((  loader['data'], loader['indices'], loader['indptr']),
                         shape = loader['shape'])
    if not is_sparse: m = m.todense() # dense matrix
    return m



def write(x,y,output_file="DATA.OUT",comment=None):
  fo = open(output_file,"w")
  if comment is not None: fo.write("# "+comment+"\n")
  for (ix,iy) in zip(x,y):
    fo.write(str(ix)+"    "+str(iy)+"\n")
  fo.close()



def save_sparse_pairs(filename,pairs):
  """Saves pairs of tuple and sparse matrix in a folder"""
  fs.rmdir(filename) # remove folder
  fs.mkdir(filename) # create folder
  fo = open(filename+"/files.txt","w") # names of the files
  for (p,m) in pairs: # loop over the pairs
    name = ""
    for ip in p: name += str(ip)+"_" # create a name
    name += ".npz"
    save_sparse_csr(filename+"/"+name,csr_matrix(m)) # save matrix
    fo.write(name+"\n") # save name of the file
  fo.close()


def read_sparse_pairs(filename,is_sparse=True):
  """Saves pairs of tuple and sparse matrix in a folder"""
  names = open(filename+"/files.txt").readlines() # names of the files
  names = [name.replace("\n","") for name in names] # remove \n
  pairs = [] # empty list
  for name in names: # loop over the pairs
    m = load_sparse_csr(filename+"/"+name) # read matrix
    if not is_sparse: m = m.todense() # dense matrix
    v = name.split("_") # slpit in numbers
    v = np.array([int(v[i]) for i in range(len(v)-1)]) # set as array
    pairs.append((v,m)) # append this pair
  return pairs





def load(input_file):
    """Load the hamiltonian"""
    with open(input_file, 'rb') as input:
      return pickle.load(input)

def save(self,output_file):
  """ Write an object"""
  with open(output_file, 'wb') as output:
    pickle.dump(self, output, pickle.HIGHEST_PROTOCOL)




def writefile(original,write=True,filename="FILE.OUT"):
  def wrapper(target):
      out = target(*args,**kwargs)
      if write: np.savetxt(filename,np.array(out).T)
      return out
  return wrapper




# this is a special library to launch parallel calculations using slurm

import dill as pickle
import os
from . import filesystem as fs
import signal
import subprocess

srcpath = os.path.dirname(os.path.realpath(__file__))+"/.." 

#pickle.settings['recurse'] = True

def pcall(fin,xs,batch_size=1,**kwargs):
    """Wrapper to allow for a batch size"""
    if batch_size==1: return pcall_single(fin,xs,**kwargs)
    else: 
        nx = len(xs) # number of xs
        xsn = [] # empty list
        o = []
        for i in range(len(xs)):
            o.append(xs[i]) # store
            if i%batch_size==0: # reached the limit
                xsn.append(o) # store
                o = [] # reset
        def fnew(y): return [fin(x) for x in y] # call this batch
        outs = pcall_single(fnew,xsn,**kwargs) # call the inputs
        out = []
        for o in outs: out += o # add
        return out


def pcall_single(fin,xs,time=10,memory=5000,error=None,
    return_mode="list"):
    """Run a parallel calculation with slurm"""
    n = len(xs) # number of calculations
    f = lambda x: fin(x)
    main = "import dill as pickle\nimport os\n"
    main += "import sys ; sys.path.append('"+srcpath+"')\n"
    main += "try: ii = int(os.environ['SLURM_ARRAY_TASK_ID'])\n"
    main += "except: ii = 0\n"
    main += "f = pickle.load(open('function.obj','rb'))\n"
    main += "v = pickle.load(open('array.obj','rb'))\n"
    main += "folder = 'folder_'+str(ii)\n"
    main += "os.system('mkdir '+folder)\n"
    main += "os.chdir(folder)\n"
    main += "pwd = os.getcwd()\n"
#    main += "out = f(v[ii])\n"
    main += "try: out = f(v[ii])\n"
    main += "except: out = None\n"
    main += "os.chdir(pwd)\n"
    main += "print(out)\n"
    main += "pickle.dump(out,open('out.obj','wb'))\n"
    main += "os.system('touch DONE')\n"
    pfolder = ".parallel"
    fs.rmdir(pfolder) # create directory
    fs.mkdir(pfolder) # create directory

    pickle.dump(f,open(pfolder+"/function.obj","wb")) # write function
    pickle.dump(xs,open(pfolder+"/array.obj","wb")) # write object
    open(pfolder+"/run.py","w").write(main) # write script
    hours = str(int(time)) # hours
    mins = int((time-int(time))*60)
    mins = str(max([mins,1])) # at least 1 minute
    runsh = "#!/bin/bash\n#SBATCH -n 1\n#SBATCH -t "+str(int(time))+":"+str(mins)+":00\n"
    runsh += "#SBATCH --mem-per-cpu="+str(memory)+"\n"
    runsh += "#SBATCH --array=0-"+str(n-1)+"\n"
    runsh += "srun python run.py\n"
    open(pfolder+"/run.sh","w").write(runsh) # parallel file
    pwd = os.getcwd() # current directory 
    os.chdir(pfolder) # go to the folder
#    os.system("sbatch run.sh >> run.out") # run calculation
    out,err = subprocess.Popen(["sbatch","run.sh"],stdout=subprocess.PIPE).communicate()
    job = job_number(out) # job number
    jobkill(job) # kill the job if exiting
    os.chdir(pwd) # back to main
    import time
    from os import path
    time.sleep(0.5) # wait half a second
    while True:
        finished = True
        for i in range(n):
            if not path.exists(pfolder+"/folder_"+str(i)+"/DONE"):
                finished = False
        if finished: break
    # get all the data
    ys = []
    for i in range(n):
        folder = pfolder+"/folder_"+str(i)+"/"
        y = pickle.load(open(folder+'out.obj','rb'))
        if y is None: y = error # use this as backup variable
        ys.append(y)
    if return_mode=="list": return ys
    elif return_mode=="dict": 
      outys = dict() # dictionary
      for i in range(n): outys[x[i]] = ys[i]
      return outys # return the dictionary



def job_number(out):
    """Get the job number"""
    out = str(out)
    out = out.split("job")[1]
    out = out.split("\\n")[0]
    return int(out) # return the job


def jobkill(n):
    """Kill the job when the program is killed"""
    def killf(*args):
      subprocess.Popen(["scancel",str(n)],stdout=subprocess.PIPE).communicate()
      print("Job killed")
      exit()
    signal.signal(signal.SIGINT, killf)
    signal.signal(signal.SIGTERM, killf)





from __future__ import print_function
import numpy as np
from scipy.sparse import csc_matrix,bmat
import time

def timeit(method):
    def timed(*args, **kw):
        ts = time.time()
        result = method(*args, **kw)
        te = time.time()
        print(method.__name__, te-ts)
        return result
    return timed


# routines to calculate DC supercurrent using keldysh formalism


def floquet_selfenergy(selfgen,e,omega,n=20,less=False,delta=0.01):
  """Generate a floquet selfenergy starting from
  a function that returns selfenergies"""
  out = [[None for i in range(2*n+1)] for i in range(2*n+1)] # output
  for i in range(-n,n+1): # loop
    ebar = e+i*omega # floquet energy
    term = selfgen(ebar) # call and store
    if less: # special propagator with the < symbol
#      bfac = 1./(np.exp(ebar/delta)+1) # boltzman factor
#      term *= bfac
      if ebar<0.: term *= 0.
      else: term = -(term-term.H)
    out[i+n][i+n] = term # store
  return bmat(out) # return dense matrix


def floquet_hamiltonian(ham,trl,omega,n=20):
  """Calculate the floquet Hamiltonian"""
  out = [[None for i in range(2*n+1)] for i in range(2*n+1)] # output
  iden = np.identity(ham.shape[0],dtype=np.complex) # identity matrix
  for i in range(-n,n+1): # loop
    out[i+n][i+n] = ham - iden*i*omega # diagonal
  for i in range(-n,n): # loop
    out[i+n][i+1+n] = trl # diagonal
    out[i+1+n][i+n] = trl.H # diagonal
#  for o in out[0]: print(o.shape)
  return bmat(out) # return dense matrix
  


def floquet_tauz(m,n=20):
  """Calculate the floquet Hamiltonian"""
  # hamgen is a function that return a time dependent hamiltonian
  out = [[None for i in range(2*n+1)] for i in range(2*n+1)] # output
  for i in range(-n,n+1): # loop
    out[i+n][i+n] = m # store
  return bmat(out) # return dense matrix
  










def current(data,voltage,n=3):
  """Apply the formula from San Jose NJP"""
  omega = voltage # frequency
  self_l = data.self_l  # self energy, callable
  self_r = data.self_r  # self energy, callable
  ham = data.ham  # Hamiltonian, matrix
  delta = data.delta  # Hamiltonian, matrix
  tfreq = data.tfreq # Hamiltonian, matrix
  tauz = data.tauz  # Hamiltonian, matrix
  def intfun(e,n=n): # function to integrate
    ftauz = floquet_tauz(tauz,n=n)
    fh = floquet_hamiltonian(ham,tfreq,omega,n=n) # floquet hamiltonian
# identity matrix
    iden = np.matrix(np.identity(fh.shape[0],dtype=np.complex)) 
    sr = floquet_selfenergy(self_r,e,omega,n=n,less=False) 
    sl = floquet_selfenergy(self_l,e,omega,n=n,less=False) 
    sR_less = floquet_selfenergy(self_r,e,omega,n=n,less=True) 
    sL_less = floquet_selfenergy(self_l,e,omega,n=n,less=True) 
    gr = (iden*(e+1j*delta) - fh - sr - sl).I # retarded green function
    gless = gr*(sL_less+sR_less)*gr.H # less green function
    val = (gr*sL_less + gless*sl.H)*ftauz
    curr = val.trace()[0,0] # add contribution
    return curr.real # return real part
  nglobal = n # set the global n 
  # function to decide the n to use afterwards
  def deciden(e,n=n,r0=-1000):
    r1 = intfun(e,n) # call the function
    dr = np.abs(r0-r1)/(np.abs(r0)+np.abs(r1)) # difference
    if dr<0.01: 
      return n
    else: return deciden(e,n=n+1,r0=r1) # recall
  nnew = deciden(voltage/2.,n=n) # update n
  def convfun(e,n=nnew,r0=-1000):
    """Converged function"""
    r1 = intfun(e,n) # call the function
    dr = np.abs(r0-r1)/(np.abs(r0)+np.abs(r1)) # difference
    if dr<0.01: 
      print("Converged with ",n,r1)
      return r1
    else: return convfun(e,n=n+1,r0=r1) # recall
  print(convfun(0.2,n=4))
#  exit()
  from scipy.integrate import quad
  xs = np.linspace(0.,omega,60) # interval
  ys = [convfun(x) for x in xs] # integrand
#  from parallel import pcall
#  ys = pcall(intfun,xs,cores=5) # return ys
  np.savetxt("TEST_"+str(voltage)+".OUT",np.matrix([xs,ys]).T)
  return np.trapz(ys,x=xs)
#  return np.sum(ys)/20
  curr,err = quad(convfun,0.0,omega,limit=10,epsrel=0.01)
  print(curr,err)
  return curr.real



def calculate_current(ht,v=0.01,delta=0.01):
  """Calculate the current using aheterostructure"""
  if ht.block_diagonal: raise # not implemented
  ###########################################################
  # time independent part of central Hamiltonian
  ham = [[None for i in range(3)] for j in range(3)]  
  ham[0][0] = ht.left_intra
  ham[1][1] = ht.central_intra
  ham[2][2] = ht.right_intra
  ham = bmat(ham).todense() # dense matrix
  # get the Nambu tauz
  from .hamiltonians import get_nambu_tauz
  from .hamiltonians import project_electrons as eproj
  from .hamiltonians import project_holes as hproj
  tauz = get_nambu_tauz(ham,has_eh=True) # Nambu tauz
  # now calculate the time dependent coupling (positive frequency)
  tfreq = [[None for i in range(3)] for j in range(3)]  
  tfreq[0][0] = ht.left_intra*0.0
  tfreq[1][1] = ht.central_intra*0.0
  tfreq[2][2] = ht.right_intra*0.0
  tfreq[0][1] = eproj(ht.left_coupling.H) # project on electrons
  tfreq[1][0] = hproj(ht.left_coupling) # project on holes
  tfreq[1][2] = eproj(ht.right_coupling) # project on electrons
  tfreq[2][1] = hproj(ht.right_coupling.H) # project on holes
  tfreq = bmat(tfreq)
  class Data(): pass
  data =  Data() # create data
  data.tauz = tauz
  data.ham = ham
  data.tfreq = tfreq
  def m2block(m,i=0):
    out = [[None for i in range(3)] for j in range(3)]  
    out[0][0] = np.zeros(ht.left_intra.shape)
    out[1][1] = np.zeros(ht.central_intra.shape)
    out[2][2] = np.zeros(ht.right_intra.shape)
    out[i][i] = m
    return bmat(out)
  lgen = lambda e: ht.get_selfenergy(e,lead=0,pristine=True)
  rgen = lambda e: ht.get_selfenergy(e,lead=1,pristine=True)
  data.self_l = lambda e: m2block(lgen(e),i=0)
  data.self_r = lambda e: m2block(rgen(e),i=2)
  data.delta = delta
  return current(data,v) # return current






# generates diffeent types of klist

import numpy as np
import scipy.linalg as lg
from . import geometry


def get_klist(g,ns,nk=100):
    """Return a klist from a list of names"""
    # generate dictionary
    kdict = dict()
    kdict["G"] = [0.,0.,0.]
    if np.abs(g.a1.dot(g.a2))<0.001: # square lattice
      kdict["X"] = [0.5,0.,0.]
      kdict["Y"] = [0.,0.5,0.]
      kdict["M"] = [0.5,0.5,0.]
    else: # triangular lattice
      angle = py_ang(g.a1,g.a2) # angle between vectors
      kdict["M1"] = [1./2.,1./2.,0.]
      kdict["M2"] = [0.,1./2.,0.]
      kdict["M3"] = [1./2.,0.,0.]
      if np.abs(angle - np.pi*2./3.)<0.001: # first type
        kdict["K"] = [1./3.,1./3.,0.]
        kdict["K'"] = [2./3.,2./3.,0.]
        kdict["M"] = [1./2.,0.,0.]
      else: # second type
        kdict["K"] = [1./3.,-1./3.,0.]
        kdict["K'"] = [2./3.,-2./3.,0.]
        kdict["M"] = [1./2.,0.,0.]
    # create the different vertex
    kv = np.array([kdict[n] for n in ns]) # get the different ones
    # create the different k-points
    ks = [] # empty list
    kinds = [0] # initial one
    for i in range(len(ns)-1): # loop over vertex
        kn = kv[i+1]-kv[i] # vector linking the two
        k0 = kv[i] # first vector
        knorm = np.sqrt(kn.dot(kn)) # norm
        nki = int(nk*knorm) # number of points
        kinds.append(nki+sum(kinds)) # store
        dk = np.linspace(0.,1.,nki,endpoint=False) # create increment
        for ik in dk:
            ks.append(k0 + ik*kn) # new vector
    ks.append(kv[len(ns)-1]) # last one
    fbl = open("BANDLINES.OUT","w")
    for i in range(len(ns)): 
        fbl.write(str(kinds[i])+" "+ns[i]+"\n")
    return ks # return vector





def py_ang(v1, v2):
  """ Returns the angle in radians between vectors 'v1' and 'v2'    """
  cosang = np.dot(v1, v2)
  sinang = lg.norm(np.cross(v1, v2))
  return np.arctan2(sinang, cosang)







def default_kpath(g,nk=400):
  """ Input is geometry"""
  if g.dimensionality==0: return [0.] # return gamma point
  elif g.dimensionality==1: 
    return [np.array([k,0,0]) for k in np.linspace(0,1,nk)] # normal path
  elif g.dimensionality > 1:
    b1 = np.array([1.,0.,0.])
    b2 = np.array([0.,1.,0.])
    (G1,G2) = geometry.get_reciprocal2d(g.a1,g.a2)
    bm = b1 + b2 # wave-vector along M
    def py_ang(v1, v2):
      """ Returns the angle in radians between vectors 'v1' and 'v2'    """
      cosang = np.dot(v1, v2)
      sinang = lg.norm(np.cross(v1, v2))
      return np.arctan2(sinang, cosang)
    angle = py_ang(g.a1,g.a2) # angle between vectors
    if np.abs(angle - np.pi*2./3.)<0.001:  
      bm = np.array([1.,1.,0.]) # temporal fix
#    else np.abs(angle - np.pi*2./6.):  
    else:
      bm = np.array([1.,-1.,0.]) # temporal fix
#    print("Path along",bm)
#    b2 = np.array([.5,np.sqrt(3)/2])
#    b2 = np.array([0.,-1.])
    fk = open("KPOINTS_BANDS.OUT","w")  
#    fk.write(str(nk)+"\n") # number of kpoints
    k = np.array([0.,0.,0.]) # old kpoint
    kout= []
    for i in range(nk):
      k += bm /(nk) # move kpoint 
      fk.write(str(k[0])+"   "+str(k[1])+"\n    ")
      kout.append(k.copy()) # store in array
    fk.close()
    # write bandlines
    fbl = open("BANDLINES.OUT","w")
    if np.abs(g.a1.dot(g.a2))<0.001: # square lattice
      fbl.write("0   \Gamma\n")
      fbl.write(str(nk/2)+"   M\n")
      fbl.write(str(nk)+"   \Gamma\n")

    else: # triangular lattice
      fbl.write("0   \Gamma\n")
      fbl.write(str(nk/3)+"   K\n")
      fbl.write(str(nk/2)+"   M\n")
      fbl.write(str(2*nk/3)+"   K'\n")
      fbl.write(str(nk)+"   \Gamma\n")
    fbl.close()
    return kout


default = default_kpath # for compatibility

# def full_bz(g)


# def gmxg(nk=100):



def gen_default(k):
  """ Return a function which generates the path"""
  b1 = np.array([1.,0.])
  b2 = np.array([0.,1.])
  return k*(b1+b2) # return kpoint 


def path_GKMKG(g,nk):
  """Generate a path G-K-M-K'-G"""
  a1 = g.a1 # vector
  a2 = g.a2 # vector
  raise



def write_klist(kl,output_file="klist.in"):
  """ Writes a set of 2D kpoints in a file"""
  fk = open(output_file,"w")  
  fk.write(str(len(kl))+"\n") # number of kpoints
  for k in kl:
    fk.write(str(k[0])+"   "+str(k[1])+"\n    ")
  fk.close()


def custom_klist(kp = None,nk=100,write=True,scale=True):
  if kp==None:
    kp = [[0.,0.,0.],[.5,.0,0.],[.5,.5,0.],[.0,.0,0.]]  # define points
  kp = [np.array(k) for k in kp] # convert to arrays
  ks = [] # empty list of kpoints
  for i in range(len(kp)-1): # loop over pairs
    dk = kp[i+1] - kp[i]
    kk = np.sqrt(dk.dot(dk)) # norm of the vector
    if scale:
      steps = np.linspace(0.,1.,int(nk*kk),endpoint=False) # number of points
    else:
      steps = np.linspace(0.,1.,nk,endpoint=False) # number of points
    for s in steps:
      ks += [kp[i] + dk*s] # add kpoint
  if write:
    write_klist(ks)
  return ks

custom = custom_klist # same function


def kx(g,nk=400):
  """ Input is geometry"""
  if g.dimensionality == 2:
    b1 = np.array([1.,0.])
    b2 = np.array([0.,1.])
    fk = open("klist.in","w")  
    fk.write(str(nk)+"\n") # number of kpoints
    k = -b1/2 # old kpoint
    kout= []
    for i in range(nk):
      k += (b1) /(nk) # move kpoint 
      fk.write(str(k[0])+"   "+str(k[1])+"\n    ")
      kout.append(k.copy()) # store in array
    fk.close()
    # write bandlines
    fbl = open("BANDLINES.OUT","w")
    fbl.write("0   X_1\n")
    fbl.write(str(nk/2)+"   \Gamma\n")
    fbl.write(str(nk)+"   X_1\n")
    fbl.close()
  return kout # return klist


def tr_path(nk=100,d=20,write=True):
  """ Creates the special path to calculate the Z2 invariant"""
  # d is the number of divisions
  w = 1.0/8 # heigh of the path
  ks = [] # initialice list
  dk = 1./nk 
  # full path
  ks += [[il,.5/d] for il in np.arange(-.5-dk,.5/d,dk)] 
  ks += [[.5/d,il] for il in np.arange(.5/d,0-dk,-dk)] 
  ks += [[il,0.-dk] for il in np.arange(.5/d,(d-1)*.5/d,dk)] 
  ks += [[(d-1)*.5/d,il] for il in np.arange(0.-dk,.5/d,dk)] 
  ks += [[il,.5/d] for il in np.arange((d-1)*.5/d,.5+dk,dk)] 
  ks += [[.5+dk,il] for il in np.arange(.5/d,(d-1)*.5/d,dk)] 
  ks2 = [[-k[0],.5-k[1]] for k in ks]
  ks = ks + ks2 # sum the two lists
  if write:
    f = open("klist.in","w")
    f.write(str(len(ks))+"\n") # number of kpoints
    for k in ks:
      f.write(str(k[0]) + "  ")
      f.write(str(k[1]) + "\n")
    f.close()



def tr_klist(nk=100,d=20):
  """ Creates the special path to calculate the Z2 invariant"""
  # d is the number of divisions
  w = 1.0/8 # heigh of the path
  lks = [] # initialice list
  dk = 1./nk 
  # full path
  class ks_class: pass
  ks = ks_class() # create object
  lks += [[il,.5/d] for il in np.arange(-.5-dk,.5/d,dk)] 
  lks += [[.5/d,il] for il in np.arange(.5/d,0-dk,-dk)] 
  lks += [[il,0.-dk] for il in np.arange(.5/d,(d-1)*.5/d,dk)] 
  lks += [[(d-1)*.5/d,il] for il in np.arange(0.-dk,.5/d,dk)] 
  lks += [[il,.5/d] for il in np.arange((d-1)*.5/d,.5+dk,dk)] 
  ks.common = [[.5+dk,il] for il in np.arange(.5/d,(d-1)*.5/d,dk)] 
  lks2 = [[-k[0],.5-k[1]] for k in lks]
  ks.path1 = lks  # first path
  ks.path2 = lks2  # second path
  return ks




def default_v2(g,nk=400):
  """ Input is geometry"""
  if g.dimensionality == 2:
    b1 = np.array([1.,0.])
    b2 = np.array([0.,1.])
    fk = open("klist.in","w")  
    fk.write(str(nk)+"\n") # number of kpoints
    k = np.array([0.,0.]) # old kpoint
    kout= []
    for i in range(nk):
      k += (b1-b2) /(nk) # move kpoint 
      fk.write(str(k[0])+"   "+str(k[1])+"\n    ")
      kout.append(k.copy()) # store in array
    fk.close()
    # write bandlines
    fbl = open("BANDLINES.OUT","w")
    fbl.write("0   \Gamma\n")
    fbl.write(str(nk/3)+"   K\n")
    fbl.write(str(nk/2)+"   M\n")
    fbl.write(str(2*nk/3)+"   K'\n")
    fbl.write(str(nk)+"   \Gamma\n")
    fbl.close()
  else: raise # not implemented





def kmesh(dimensionality,nk=10,nsuper=1):
  """Return a mesh of k-points for a certain dimensionality"""
  from .checkclass import number2array
  kp = []
  if nk==1 or dimensionality==0: return [[0.,0.,0.]]
  if dimensionality==1:
    for k1 in np.linspace(0.,nsuper,nk,endpoint=False):
      kp.append([k1,0.,0.]) # store
  elif dimensionality==2:
    nk = number2array(nk) # return an array
    for k1 in np.linspace(0.,nsuper,nk[0],endpoint=False):
      for k2 in np.linspace(0.,nsuper,nk[1],endpoint=False):
        kp.append([k1,k2,0.]) # store
  elif dimensionality==3:
    nk = number2array(nk) # return an array
    for k1 in np.linspace(0.,nsuper,nk[0],endpoint=False):
      for k2 in np.linspace(0.,nsuper,nk[1],endpoint=False):
        for k3 in np.linspace(0.,nsuper,nk[2],endpoint=False):
          kp.append([k1,k2,k3]) # store
  else: raise
  kp = [np.array(k) for k in kp] # to array
  return kp




def label2k(g,kl):
    """Given a kpoint label, return the kpoint"""
    if kl=="G": return [0.,0.,0.]
    elif kl=="M": return [0.5,0.,0.]
    elif kl=="K":
        from .kpointstk.locate import target_moduli
        from .kpointstk.mapping import unitary
        vm = np.sqrt(3) # moduli
        k = g.k2K(target_moduli(unitary(g.b1),unitary(g.b2),vm))
        return k*2/(3.*np.sqrt(3))
    elif kl=="K'": return -label2k(g,"K")
    elif kl=="M1": return [.5,.0,.0]
    elif kl=="M2": return [.0,.5,.0]
    elif kl=="M3": return [.5,.5,.0]
    elif kl=="X": return [.5,.5,.0]
    else: raise





def get_kpath_labels(g,ks,**kwargs):
    """Return the k-path"""
    kps = [label2k(g,k) for k in ks] # get the kpoints
    from .kpointstk.locate import k2path
    return k2path(g,kps,**kwargs) # closest path


def get_kpath(g,kpath=None,**kwargs):
    """Return a kpath"""
    if kpath is None: return default(g,**kwargs)
    elif type(kpath[0])==str: return get_kpath_labels(g,kpath,**kwargs)
    else: return kpath # assume is a valid list of vectors







#!/mnt/extradrive/apps/anaconda3/bin/python

import numpy as np
import numpy 
from numba import jit



@jit(nopython=True)
def wave_jit(x,y,centers,heights,facx,facy,z):
  for i in range(len(centers)):
    c = centers[i]
    h = heights[i]
    r = facx*(x-c[0])**2+facy*(y-c[1])**2 # renormalized gaussian       
    z += h*np.exp(-(r))
  return z




def compute_interpolation(centers=[[0.,0.,0.]], heights=[10.0],name="",
        nx=600,ny=600,smooth=2,smooth_nxny=False,xcut=None,ycut=None,
        dx=None,dy=None):
    """ Creates a set of gaussians at that position and with a height"""
    if len(heights)<len(centers): raise
    if smooth_nxny:
        if xcut is not None:
            facx = nx/float(xcut)
        else:
            facx = nx/np.max(np.abs(np.array(centers)[:,0]))
        if ycut is not None:
            facy = ny/float(ycut)
        else:
            facy = ny/np.max(np.abs(np.array(centers)[:,1]))
        facx = facx/smooth
        facy = facy/smooth
    else:
      fac = 1./smooth # factor to mix the gaussians
      facx = fac
      facy = fac
    def wave(x,y):
        z = x*0.0 + y*0.0
        return wave_jit(x,y,np.array(centers),np.array(heights),facx,facy,z)
    xs = [c[0] for c in centers] # get x coordinates
    ys = [c[1] for c in centers] # get y coordinates
    if dx is None: dx = (np.max(xs) - np.min(xs))/10
    else: dx = float(dx)
    if dy is None: dy = (np.max(ys) - np.min(ys))/10
    else: dy = float(dy)
    x = np.linspace(min(xs)-dx,max(xs)+dx,nx)
    y = np.linspace(min(ys)-dy,max(ys)+dy,ny)
    xout = []
    yout = []
    zout = []
    for ix in x:
      z = wave(ix,y)
      for (iy,iz) in zip(y,z):
          xout.append(ix)
          zout.append(iz)
          yout.append(iy)
    return xout,yout,zout



def atomic_interpolation(x,y,z,xcut=None,ycut=None,**kwargs):
    col = 2
    m = np.array([x,y,z])
    if xcut is not None: # given value
        xc = float(xcut)
        m = m[:,np.abs(m[0,:])<xc]
    if ycut is not None: # given value
        yc = float(ycut)
        m = m[:,np.abs(m[1,:])<yc]
    centers = [[m[0,i],m[1,i],0.] for i in range(len(m[0]))]
    heights = m[col,:]
    return compute_interpolation(centers=centers,heights=heights,**kwargs)
  





from __future__ import print_function
import numpy as np
import scipy.linalg as lg
from . import multicell
from . import algebra
from numba import jit
from .greentk.rg import green_renormalization
from .greentk.selfenergy import bloch_selfenergy
from .greentk.kchain import green_kchain
from .greentk.kchain import get1dhamiltonian
from .greentk.kchain import green_kchain_evaluator


use_fortran = False

try: from .gauss_invf90 import gauss_inv as ginv
except: 
  pass


class gf_convergence():
   """ Class to manage the convergence  options
   of the green functions """
   optimal = False
   refinement = False
   guess = True # use old green function
   def __init__(self,mode):
     if mode=="fast":   # fast mode,used for coule to finite systems 
       self.eps = 0.001
       self.max_error = 1.0
       self.num_rep = 10
       self.mixing = 1.0
     if mode=="lead":
       self.eps = 0.001
       self.max_error = 0.00001
       self.num_rep = 3
       self.mixing = 0.8
     if mode=="hundred":  
       self.eps = 0.001
       self.max_error = 1.0
       self.num_rep = 100
       self.mixing = 1.0



def dyson(intra,inter,energy=0.0,gf=None,is_sparse=False,initial = None):
  """ Solves the dyson equation for a one dimensional
  system with intra matrix 'intra' and inter to the nerest cell
  'inter'"""
  # get parameters
  if gf is None: gf = gf_convergence("lead")
  mixing = gf.mixing
  eps = gf.eps
  max_error = gf.max_error
  num_rep = gf.num_rep
  optimal = gf.optimal
  try:
    intra = intra.todense()
    inter = inter.todense()
  except:
    a = 1
  if initial is None:  # if green not provided. initialize at zero
    from numpy import zeros
   
    g_guess = intra*0.0j
  else:
    g_guess = initial
  # calculate using fortran
  if optimal:
    print("Fortran dyson calculation")
    from .green_fortran import dyson  # import fortran subroutine
    (g,num_redo) = dyson(intra,inter,energy,num_rep,mixing=mixing,
               eps=eps,green_guess=g_guess,max_error=max_error)
    print("      Converged in ",num_redo,"iterations\n")
    from numpy import matrix
    g = matrix(g)
  # calculate using python
  if not optimal:
    g_old = g_guess # first iteration
    iden = np.matrix(np.identity(len(intra),dtype=complex)) # create identity
    e = iden*(energy+1j*eps) # complex energy
    while True: # loop over iterations
      self = inter@g_old@inter.H # selfenergy
      g = (e - intra - self).I # dyson equation
      if np.max(np.abs(g-g_old))<gf.max_error: break
      g_old = mixing*g + (1.-mixing)*g_old # new green function
  if is_sparse: 
    from scipy.sparse import csc_matrix
    g = csc_matrix(g)
  return g











def dos_infinite(intra,inter,energies=[0.0],num_rep=100,
                      mixing=0.7,eps=0.0001,green_guess=None,max_error=0.0001):
   """ Calculates the surface density of states by using a 
    green function approach"""
   dos = [] # list with the density of states
   iden = np.matrix(np.identity(len(intra),dtype=complex)) # create idntity
   for energy in energies: # loop over energies
     # right green function
     gr = dyson(intra,inter,energy=energy,num_rep=num_rep,mixing=mixing,
          eps=eps,green_guess=green_guess,max_error=max_error)
     # left green function
     gl = dyson(intra,inter.H,energy=energy,num_rep=num_rep,mixing=mixing,
          eps=eps,green_guess=green_guess,max_error=max_error)
     # central green function
     selfl = inter.H@gl@inter # left selfenergy
     selfr = inter@gr@inter.H # right selfenergy
     gc = energy*iden -intra -selfl -selfr # dyson equation for the center
     gc = gc.I # calculate inverse
     dos.append(-algebra.trace(gc).imag)  # calculate the trace of the Green function
   return dos




def dos_semiinfinite(intra,inter,energies=np.linspace(-1.0,1.0,100),num_rep=100,
                      mixing=0.7,eps=0.0001,green_guess=None,max_error=0.0001):
   """ Calculates the surface density of states by using a 
    green function approach"""
   dos = [] # list with the density of states
   for energy in energies: # loop over energies
#     gf = dyson(intra,inter,energy=energy,num_rep=num_rep,mixing=mixing,
     gb,gf = green_renormalization(intra,inter,energy=energy,delta=delta)
     dos.append(-algebra.trace(gf).imag)  # calculate the trace of the Green function
   return energies,dos









def dos_heterostructure(hetero,energies=[0.0],num_rep=100,
                      mixing=0.7,eps=0.0001,green_guess=None,max_error=0.0001):
   """ Calculates the density of states 
       of a heterostructure by a  
    green function approach, input is a heterostructure class"""
   dos = [] # list with the density of states
   iden = np.matrix(np.identity(len(intra),dtype=complex)) # create idntity
   for energy in energies: # loop over energies
     # right green function
     intra = hetero.right_intra
     inter = hetero.right_inter
     gr = dyson(intra,inter,energy=energy,num_rep=num_rep,mixing=mixing,
          eps=eps,green_guess=green_guess,max_error=max_error)
     # left green function
     intra = hetero.right_intra
     inter = hetero.right_inter
     gl = dyson(intra,inter,energy=energy,num_rep=num_rep,mixing=mixing,
          eps=eps,green_guess=green_guess,max_error=max_error)
     # central green function
     selfl = inter.H@gl@inter # left selfenergy
     selfr = inter@gr@inter.H # right selfenergy
     gc = energy*iden -intra -selfl -selfr # dyson equation for the center
     gc = gc.I # calculate inverse
     dos.append(-algebra.trace(gc).imag)  # calculate the trace of the Green function
   return dos



def read_matrix(f):
  """Read green function from a file"""
  m = np.genfromtxt(f)
  d = int(max(m.transpose()[0]))+1 # dimension of the green functions
  g = np.matrix([[0.0j for i in range(d)] for j in range(d)]) # create matrix
  for r in m:
    i = int(r[0])
    j = int(r[1])
    ar = r[2]
    ai = r[3]
    g[i,j] = ar +1j*ai # store element
  return g # return green function



def write_matrix(f,g):
  """Write green function from a file"""
  fw = open(f,"w") # open file to write
  n = len(g) # dimension of the matrix
  for i in range(n):
    for j in range(n):
      fw.write(str(i)+"  ")
      fw.write(str(j)+"  ")
      fw.write(str(g[i,j].real)+"  ")
      fw.write(str(g[i,j].imag)+"\n")
  fw.close()   # close file


# detect non vanishing elements of a matrix
def nv_el(m):
  """ get the non vanishing elments of a matrix"""
  from scipy.sparse import csc_matrix as csc
  mc = csc(m) # to coo_matrixi
  mc.eliminate_zeros()
  mc = mc.tocoo()
  data = mc.data # get data
  col = mc.col # get column index
  row = mc.row # get row index
  nv = []
  nt=len(data)
  for i in range(nt):
   # save the nonvanishing values
   nv.append([row[i]+1,col[i]+1,data[i].real,data[i].imag])
  return nv


def write_sparse(f,g):
  """ Write a sparse matrix in a file"""
  fw = open(f,"w") # open the file
  fw.write("# dimension = "+str(g.shape[0])+"\n")
  nv=nv_el(g)
  for iv in range(len(nv)):
    fw.write(str(int(nv[iv][0]))+'   ')
    fw.write(str(int(nv[iv][1]))+'   ')
    fw.write('{0:.8f}'.format(float(nv[iv][2]))+'   ')
    fw.write('{0:.8f}'.format(float(nv[iv][3]))+'   ')
    fw.write('  !!!  i  j   Real   Imag\n')
  fw.close()




def read_sparse(f,sparse=True):
  """Read green function from a file"""
  l = open(f,"r").readlines()[0] # first line
  d = int(l.split("=")[1])
  m = np.genfromtxt(f)
  if not sparse:
# create matrix  
    g = np.matrix([[0.0j for i in range(d)] for j in range(d)]) 
    for r in m:
      i = int(r[0])-1
      j = int(r[1])-1
      ar = r[2]
      ai = r[3]
      g[i,j] = ar +1j*ai # store element
  if sparse:
    from scipy.sparse import coo_matrix
    g = coo_matrix([[0.0j for i in range(d)] for j in range(d)]) 
    row = np.array([0 for i in range(len(m))])
    col = np.array([0 for i in range(len(m))])
    data = np.array([0j for i in range(len(m))])
    for i in range(len(m)):
      r = m[i]
      row[i] = int(r[0])-1
      col[i] = int(r[1])-1
      ar = r[2]
      ai = r[3]
      data[i] = ar +1j*ai # store element
    g.col = col
    g.row = row
    g.data = data
  return g # return green function





def gauss_inverse(m,i=0,j=0,test=False):
  try: from .gauss_invf90 import gauss_inv as ginv
  except: 
    test = True # Ups, this might blow up
  """ Calculates the inverso of a block diagonal
      matrix """
  if test: # check whether the inversion worked 
    return block_inverse(m,i=i,j=j)
  nb = len(m) # number of blocks
  ca = [None for ii in range(nb)]
  ua = [None for ii in range(nb-1)]
  da = [None for ii in range(nb-1)]
  for ii in range(nb): # diagonal part
    ca[ii] = m[ii][ii]
  for ii in range(nb-1):
    ua[ii] = m[ii][ii+1]
    da[ii] = m[ii+1][ii]
  # in case you use the -1 notation of python
  if i<0: i += nb 
  if j<0: j += nb 
  # now call the actual fortran routine
  mout = ginv(ca,da,ua,i+1,j+1)
  mout = np.matrix(mout)
  return mout




def block_inverse(m,i=0,j=0):
    """ Calculate a certain element of the inverse of a block matrix"""
    from scipy.sparse import csc_matrix,bmat
    nb = len(m) # number of blocks
    if i<0: i += nb 
    if j<0: j += nb 
    mt = [[None for ii in range(nb)] for jj in range(nb)]
    for ii in range(nb): # diagonal part
      mt[ii][ii] = csc_matrix(m[ii][ii])
    for ii in range(nb-1):
      mt[ii][ii+1] = csc_matrix(m[ii][ii+1])
      mt[ii+1][ii] = csc_matrix(m[ii+1][ii])
    mt = bmat(mt).todense() # create dense matrix
    # select which elements you need
    ilist = [m[ii][ii].shape[0] for ii in range(i)] 
    jlist = [m[jj][jj].shape[1] for jj in range(j)] 
    imin = sum(ilist)
    jmin = sum(jlist)
    mt = mt.I # calculate inverse
    imax = imin + m[i][i].shape[0]
    jmax = jmin + m[j][j].shape[1]
    mo = [ [mt[ii,jj] for jj in range(jmin,jmax)] for ii in range(imin,imax) ] 
    mo = np.matrix(mo)
    return mo










def green_surface_cells(gs,hop,ons,delta=1e-2,e=0.0,n=0):
    """Compute the surface Green's function for several unit cells"""
    hopH = algebra.H(hop) # Hermitian
    ez = (e+1j*delta)*np.identity(ons.shape[0]) # energy
    gt = np.zeros(ons.shape[0],dtype=np.complex) # energy
    sigmar = hop@gs@algebra.H(hop) # of the infinite right part
    out = []
    for i in range(n):
      sigmal = algebra.H(hop)@gt@hop # selfenergy
      # couple infinite right to finite left
      gemb = algebra.inv(ez - ons - sigmal- sigmar) # full dyson equation
      # compute surface spectral function of the left block only
      gt = algebra.inv(ez - ons - sigmal) # return Dyson equation
      out.append(gemb) # store this green's function
    return out # return green's functions





def interface(h1,h2,k=[0.0,0.,0.],energy=0.0,delta=0.01):
  """Get the Green function of an interface"""
  from scipy.sparse import csc_matrix as csc
  from scipy.sparse import bmat
  gs1,sf1 = green_kchain(h1,k=k,energy=energy,delta=delta,
                   only_bulk=False,reverse=True) # surface green function 
  gs2,sf2 = green_kchain(h2,k=k,energy=energy,delta=delta,
                   only_bulk=False,reverse=False) # surface green function 
  #############
  ## 1  C  2 ##
  #############
  # Now apply the Dyson equation
  (ons1,hop1) = get1dhamiltonian(h1,k,reverse=True) # get 1D Hamiltonian
  (ons2,hop2) = get1dhamiltonian(h2,k,reverse=False) # get 1D Hamiltonian
  havg = (algebra.dagger(hop1) + hop2)/2. # average hopping
  ons = bmat([[csc(ons1),csc(havg)],[csc(havg.H),csc(ons2)]]) # onsite
  self2 = bmat([[csc(ons1)*0.0,None],[None,csc(hop2@sf2@hop2.H)]])
  self1 = bmat([[csc(hop1@sf1@hop1.H),None],[None,csc(ons2)*0.0]])
  # Dyson equation
  ez = (energy+1j*delta)*np.identity(ons1.shape[0]+ons2.shape[0]) # energy
  ginter = (ez - ons - self1 - self2).I # Green function
  # now return everything, first, second and hybrid
  return (gs1,sf1,gs2,sf2,ginter)


def interface_multienergy(h1,h2,k=[0.0,0.,0.],energies=[0.0],delta=0.01,
        dh1=None,dh2=None):
  """Get the Green function of an interface"""
  from scipy.sparse import csc_matrix as csc
  from scipy.sparse import bmat
  fun1 = green_kchain_evaluator(h1,k=k,delta=delta,hs=None,
                   only_bulk=False,reverse=True) # surface green function 
  fun2 = green_kchain_evaluator(h2,k=k,delta=delta,hs=None,
                   only_bulk=False,reverse=False) # surface green function 
  out = [] # output
  for energy in energies: # loop
    gs1,sf1 = fun1(energy)
    gs2,sf2 = fun2(energy)
    #############
    ## 1  C  2 ##
    #############
    # Now apply the Dyson equation
    (ons1,hop1) = get1dhamiltonian(h1,k,reverse=True) # get 1D Hamiltonian
    (ons2,hop2) = get1dhamiltonian(h2,k,reverse=False) # get 1D Hamiltonian
    havg = (algebra.dagger(hop1) + hop2)/2. # average hopping
    if dh1 is not None: ons1 = ons1 + dh1
    if dh2 is not None: ons2 = ons2 + dh2
    ons = bmat([[csc(ons1),csc(havg)],[csc(havg.H),csc(ons2)]]) # onsite
    self2 = bmat([[csc(ons1)*0.0,None],[None,csc(hop2@sf2@hop2.H)]])
    self1 = bmat([[csc(hop1@sf1@hop1.H),None],[None,csc(ons2)*0.0]])
    # Dyson equation
    ez = (energy+1j*delta)*np.identity(ons1.shape[0]+ons2.shape[0]) # energy
    ginter = algebra.inv(ez - ons - self1 - self2) # Green function
    # now return everything, first, second and hybrid
    out.append([gs1,sf1,gs2,sf2,ginter])
  return out # return output





def surface_multienergy(h1,k=[0.0,0.,0.],energies=[0.0],reverse=True,**kwargs):
  """Get the Green function of an interface"""
  from scipy.sparse import csc_matrix as csc
  from scipy.sparse import bmat
  fun1 = green_kchain_evaluator(h1,k=k,
                   only_bulk=False,reverse=reverse,
                   **kwargs) # surface green function 
  out = [] # output
  from . import parallel
  def fp(x):
      gs1,sf1 = fun1(x)
      return [sf1,gs1]
  out = parallel.pcall(fp,energies)
#  for energy in energies: # loop
#    gs1,sf1 = fun1(energy)
#    out.append([sf1,gs1])
  return out # return output















def supercell_selfenergy(h,e=0.0,delta=0.001,nk=100,nsuper=[1,1]):
  """Calculates the selfenergy of a certain supercell """
  if nsuper==1:
      return bloch_selfenergy(h,energy=e,delta=delta,nk=nk,
              mode="renormalization")
  if h.dimensionality>2: return NotImplemented
  try:   # if two number given
    nsuper1 = nsuper[0]
    nsuper2 = nsuper[1]
  except: # if only one number given
    nsuper1 = nsuper
    nsuper2 = nsuper
#  print("Supercell",nsuper1,"x",nsuper2)
  ez = e + 1j*delta # create complex energy
  from . import dyson
  g = dyson.dyson(h,[nsuper1,nsuper2],nk,ez)
  g = np.matrix(g) # convert to matrix
  # create hamiltonian of the supercell
  from .embedding import onsite_supercell
  intrasuper = onsite_supercell(h,nsuper)
  eop = np.matrix(np.identity(len(g),dtype=np.complex))*(ez)
  selfe = eop - intrasuper - algebra.inv(g)
  return g,selfe







def green_generator(h,nk=20):
  """Returns a function capable of calculating the Green function
  at a certain energy, by explicity summing the k-dependent Green functions"""
  if h.dimensionality != 2: raise # only for 2d
  shape = h.intra.shape # shape
  hkgen = h.get_hk_gen() # get the Hamiltonian generator
  wfs = np.zeros((nk*nk,shape[0],shape[0]),dtype=np.complex) # allocate vector
  es = np.zeros((nk*nk,shape[0])) # allocate vector, energies
  ks = np.zeros((nk*nk,2)) # allocate vector, energies
  ii = 0 # counter
  for ik in np.linspace(0.,1.,nk,endpoint=False): # loop
    for jk in np.linspace(0.,1.,nk,endpoint=False): # loop
      estmp,wfstmp = algebra.eigh(hkgen([ik,jk])) # get eigens
#      estmp,wfstmp = lg.eigh(hkgen(np.random.random(2))) # get eigens
      es[ii,:] = estmp.copy() # copy
      ks[ii,:] = np.array([ik,jk]) # store
      wfs[ii,:,:] = wfstmp.transpose().copy() # copy
      ii += 1 # increase counter
  # All the wavefunctions have been calculate
  # Now create the output function
  from scipy.integrate import simps
  def getgreen(energy,delta=0.001):
    """Return the Green function"""
    zero = np.array(np.zeros(shape,dtype=np.complex)) # zero matrix
    zero = getgreen_jit(wfs,es,energy,delta,zero)
    ediag = np.array(np.identity(shape[0]))*(energy + delta*1j)
    selfenergy = ediag - h.intra - algebra.inv(zero)
    return zero,selfenergy
  return getgreen # return function

@jit(nopython=True)
def getgreen_jit(wfs,es,energy,delta,zero):
    """Jit summation of Bloch Green's function"""
    shape = wfs[0].shape
    for ii in range(len(es)): # loop over kpoints
      v = energy + delta*1j - es[ii,:] # array
      C = zero*0.0 # initilaize
      for j in range(len(v)): C[j,j] = 1./v[j]
      A = wfs[ii,:,:] # get the matrix with wavefunctions
      zero += np.conjugate(A).T@C@A # add contribution
    zero /= len(es) # normalize
    return zero




def green_operator(h0,operator=None,e=0.0,delta=1e-3,nk=10,
        gmode="adaptive"):
    """Return the integration of an operator times the Green function"""
    h = h0.copy()
    h.turn_dense()
    hkgen = h.get_hk_gen() # get generator
    iden = np.identity(h.intra.shape[0],dtype=np.complex)
    from . import klist
    ks = klist.kmesh(h.dimensionality,nk=nk) # klist
    out = 0.0 # output
    if callable(operator): # callable operator
      for k in ks: # loop over kpoints
        hk = hkgen(k) # Hamiltonian
        o0 = algebra.inv(iden*(e+1j*delta) - hk) # Green's function
        if callable(operator): o1 = operator(k)
        else: o1 = operator
        out += -np.trace(o0@o1).imag # Add contribution
      out /= len(ks) # normalize
    else:
      g = bloch_selfenergy(h,energy=e,delta=delta,mode=gmode)[0] 
      if operator is None: out = -np.trace(np.array(g)).imag
      else: out = -np.trace(np.array(g)@operator).imag
    return out



def GtimesO(g,o,k=[0.,0.,0.]):
    """Green function times operator"""
    o = algebra.todense(o) # convert to dense operator if possible
    if o is None: return g # return Green function
    elif type(o)==type(g): return g@o # return
    elif callable(o): return o(g,k=k) # call the operator
    else:
        print(type(g),type(o))
        raise






import numpy as np
import scipy.linalg as lg

def make_unitary(M):
    """Given a non-unitary matrix, make it unitary"""
    (u,s,vh) = lg.svd(M) # perform the SVD
    s = s/np.abs(s) # normalize singular values
    M = u@np.diag(s)@vh # return the unitarized matrix
    return M
#    return M/np.abs(lg.det(M))
#    (evals,evecs) = lg.eig(M) # eigenvals and eigenvecs
#    print(np.abs(evals))
##    evecs = np.conjugate(evecs).T # change of basis
#    m2 = np.array([[0.0j for i in evals] for j in evals]) # create matrix
#    for i in range(len(evals)):  
#        m2[i,i] = evals[i]/np.abs(evals[i])
##    m2 = evecs.H * m2 * evecs  # change of basis
#    m3 = np.conjugate(evecs).T @ m2 @ evecs  # change of basis
#    return m3 # return matrix




# library for response calculation

import numpy as np
import numpy.linalg as lg
from .chi_fortran import calculate_xychi
from .collinear_xychi import collinear_xychi

def chi0d(m,energies = [0.],t=0.0001):
  """ Calculate spin response"""
  e,ev = lg.eigh(m) # get eigenvalues
  ev = np.matrix(ev.transpose())
  n = np.shape(ev)
#  import check
#  check.check(ev,e)
  ct = calculate_xychi(ev,e,ev,e,energies,t)
#  return ct


def chi1d(h,energies = [0.],t=0.0001,delta=0.01,q=0.001,nk=1000,U=None,adaptive=True,ks=None):
  hkgen = h.get_hk_gen() # get the generating hamiltonian
  n = len(h.geometry.x) # initialice response
  m = np.zeros((len(energies),n*n),dtype=np.complex) # initialice
  if not adaptive: # full algorithm  
    if ks==None: ks = np.linspace(0.,1.,nk)  # create klist
    for k in ks:
#      print "Doing k=",k
      hk = hkgen(k) # fist point
      e1,ev1 = lg.eigh(hk) # get eigenvalues
      hk = hkgen(k+q) # second point
      e2,ev2 = lg.eigh(hk) # get eigenvalues
      ct = calculate_xychi(ev1.T,e1,ev2.T,e2,energies,t,delta) # contribution
      m += ct
    m = m/nk # normalice by the number of kpoints  
    ms = [m[i,:].reshape(n,n) for i in range(len(m))] # convert to matrices
  if adaptive: # adaptive algorithm 
    from .integration import integrate_matrix
    def get_chik(k): # function which returns a matrix
      """ Get response at a cetain energy"""
      hk = hkgen(k) # first point
      e1,ev1 = lg.eigh(hk) # get eigenvalues
      hk = hkgen(k+q) # second point
      e2,ev2 = lg.eigh(hk) # get eigenvalues
      ct = calculate_xychi(ev1.T,e1,ev2.T,e2,energies,t,delta) # contribution
      return ct # return response
    ms = []
    m = integrate_matrix(get_chik,xlim=[0.,1.],eps=.1,only_imag=False) # add energy
    ms = [m[i,:].reshape(n,n) for i in range(len(m))] # convert to matrices

  if U==None: # raw calculation
    return ms 
  else: # RPA calculation
    return rpachi(ms,U=U)





def chi2d(h,energies = [0.],t=0.0001,delta=0.01,q=np.array([0.001,0.0]),nk=20,U=None,ks=None,collinear=False,adaptive=False):
  hkgen = h.get_hk_gen() # get the generating hamiltonian
  n = len(h.geometry.x) # initialice response
  m = np.zeros((len(energies),n*n),dtype=np.complex) # initialice
  ##############################
  def get_chik(k):
    """Function to integrate"""
    hk = hkgen(k) # fist point
    if collinear: hk = np.matrix([[hk[2*i,2*j] for i in range(len(hk)/2)] for j in range(len(hk)/2)])  # up component
    e1,ev1 = lg.eigh(hk) # get eigenvalues
    hk = hkgen(k+q) # second point
    if collinear: hk = np.matrix([[hk[2*i+1,2*j+1] for i in range(len(hk)/2)] for j in range(len(hk)/2)])  # down component
    e2,ev2 = lg.eigh(hk) # get eigenvalues
    if collinear:
      ct = collinear_xychi(ev1.T,e1,ev2.T,e2,energies,t,delta) # contribution
    else:
      ct = calculate_xychi(ev1.T,e1,ev2.T,e2,energies,t,delta) # contribution
    return ct
  #########################
  if adaptive: # adaptive integration
    from .integration import integrate_matrix
    def get_chik_dim2(kx): # first integration
      def get_chik_dim1(ky): # redefine the function, second integration
        return get_chik(np.array([kx,ky]))
      ct = integrate_matrix(get_chik_dim1,xlim=[0.,1.],eps=delta/100.,only_imag=False) 
      return ct # return second integration
    m = integrate_matrix(get_chik_dim2,xlim=[0.,1.],eps=delta/100.,only_imag=False) # add energy
  else: # normal integration
    ks = np.linspace(0.,1.,nk)  # create klist
    for kx in ks:
      for ky in ks:
        k = np.array([kx,ky])
        ct = get_chik(k)
        m += ct
    m = m/(nk**2) # normalice by the number of kpoints  
  ms = [m[i,:].reshape(n,n) for i in range(len(m))] # convert to matrices
  if U==None: # raw calculation
    return ms 
  else: # RPA calculation
    return rpachi(ms,U=U)






def sumchi(ms):
  """ Sums all the elements of the matrix"""
  n = ms[0].shape[0]
  return np.array([np.sum(mi.reshape(n*n)) for mi in ms])


def rpachi(ms,U=0.0):
  """ Calculate the RPA spin response"""
  iden = np.matrix(np.identity(len(ms[0]),dtype=np.complex)) # identity
  msrpa = [] # list with matrices
  for m in ms:
    m = np.matrix(m) # convert to matrix
    m = (iden - U*m).I*m  # RPA formula
    msrpa.append(m) # store matrix
  return msrpa # return RPA




def collinear_chi1d(h,energies = [0.],t=0.0001,delta=0.01,q=0.001,nk=1000,U=None,adaptive=True,ks=None):
  collinear = True
  hkgen = h.get_hk_gen() # get the generating hamiltonian
  n = len(h.geometry.x) # initialice response
  m = np.zeros((len(energies),n*n),dtype=np.complex) # initialice
  if not adaptive: # full algorithm  
    if ks==None: ks = np.linspace(0.,1.,nk)  # create klist
    for k in ks:
#      print "Doing k=",k
      hk = hkgen(k) # fist point

      if collinear: hk = np.matrix([[hk[2*i,2*j] for i in range(len(hk)/2)] for j in range(len(hk)/2)])  # up component

      e1,ev1 = lg.eigh(hk) # get eigenvalues
      hk = hkgen(k+q) # second point

      if collinear: hk = np.matrix([[hk[2*i+1,2*j+1] for i in range(len(hk)/2)] for j in range(len(hk)/2)])  # down component

      e2,ev2 = lg.eigh(hk) # get eigenvalues
      ct = collinear_xychi(ev1.T,e1,ev2.T,e2,energies,t,delta) # contribution
      m += ct
    m = m/nk # normalice by the number of kpoints  
    ms = [m[i,:].reshape(n,n) for i in range(len(m))] # convert to matrices
  if adaptive: # adaptive algorithm 
    from .integration import integrate_matrix
    def get_chik(k): # function which returns a matrix
      """ Get response at a cetain energy"""
      hk = hkgen(k) # first point
      if collinear: hk = np.matrix([[hk[2*i,2*j] for i in range(len(hk)/2)] for j in range(len(hk)/2)])  # up component
      e1,ev1 = lg.eigh(hk) # get eigenvalues
      hk = hkgen(k+q) # second point
      if collinear: hk = np.matrix([[hk[2*i+1,2*j+1] for i in range(len(hk)/2)] for j in range(len(hk)/2)])  # down component
      e2,ev2 = lg.eigh(hk) # get eigenvalues
      ct = collinear_xychi(ev1.T,e1,ev2.T,e2,energies,t,delta) # contribution
      return ct # return response
    ms = []
    m = integrate_matrix(get_chik,xlim=[0.,1.],eps=.01,only_imag=False) # add energy
    ms = [m[i,:].reshape(n,n) for i in range(len(m))] # convert to matrices

  if U==None: # raw calculation
    return ms 
  else: # RPA calculation
    return rpachi(ms,U=U)













if __name__=="__main__":
  from . import geometry
  from . import hamiltonians
  g = geometry.chain()
  h = g.get_hamiltonian()
#  h.add_antiferromagnetism(.1)
  h.add_antiferromagnetism(.1)
  energies = np.linspace(-1.,1.,100)
  ms = chi1d(h,energies=energies)
  ms = sumchi(ms) # sum all the elements
  import pylab as py
#  b = h.plot_bands()
  py.plot(energies,ms.imag)
  py.plot(energies,ms.real)
  py.show()



import numpy as np
import scipy.linalg as lg

def get_fractional_function(g,center=False):
    """Get fractional coordinates"""
  #  if g.dimensionality<2: raise # stop 
    dim = g.dimensionality # dimensionality
    if dim==0: return lambda x: x
    elif dim==1: # one dimensional
      R = np.array([g.a1,[0.,1.,0.],[0.,0.,1.]]).T # transformation matrix
      if np.max(np.abs(g.a1[1:2]))>1e-6: raise
    elif dim==2: # two dimensional
      R = np.array([g.a1,g.a2,[0.,0.,1.]]).T # transformation matrix
      if np.abs(g.a1[2])>1e-6 or np.abs(g.a2[2])>1e-6: raise
    elif dim==3:
      R = np.array([g.a1,g.a2,g.a3]).T # transformation matrix
    else: raise
    g.has_fractional = True # has fractional coordinates
    L = lg.inv(R) # inverse matrix
    def f(r):
        if center: return (L@np.array(r))%1.0  # transform
        else: return L@np.array(r)  # transform
    return f




def get_fractional(g,center=False):
    dim = g.dimensionality # dimensionality
    if dim==0: return
    f = get_fractional_function(g,center=center)
    store = [f(r) for r in g.r] # empty list
    store = np.array(store) # convert to array
    # if you remove the shift the Berry Green formalism does not work
    if dim>0: g.frac_x = store[:,0]
    if dim>1: g.frac_y = store[:,1]
    if dim>2: g.frac_z = store[:,2]
    g.frac_r = store




def fractional2real(self):
    """Write real coordinates using the fractional ones"""
    if self.dimensionality==0: raise
    elif self.dimensionality==1: # 1D
      self.x = self.frac_x*self.a1[0]
    elif self.dimensionality==2: # 2D
      self.x = self.frac_x*self.a1[0] +  self.frac_y*self.a2[0]
      self.y = self.frac_x*self.a1[1] +  self.frac_y*self.a2[1]
    elif self.dimensionality==3: # 3D
      self.x = self.frac_x*self.a1[0] +  self.frac_y*self.a2[0] + self.frac_z*self.a3[0]
      self.y = self.frac_x*self.a1[1] +  self.frac_y*self.a2[1] + self.frac_z*self.a3[1]
      self.z = self.frac_x*self.a1[2] +  self.frac_y*self.a2[2] + self.frac_z*self.a3[2]
    else: raise
    self.xyz2r() # update xyz
    self.center() # center




